/*
 * Copyright (c) 2009 Xilinx, Inc.  All rights reserved.
 *
 * Xilinx, Inc.
 * XILINX IS PROVIDING THIS DESIGN, CODE, OR INFORMATION "AS IS" AS A
 * COURTESY TO YOU.  BY PROVIDING THIS DESIGN, CODE, OR INFORMATION AS
 * ONE POSSIBLE   IMPLEMENTATION OF THIS FEATURE, APPLICATION OR
 * STANDARD, XILINX IS MAKING NO REPRESENTATION THAT THIS IMPLEMENTATION
 * IS FREE FROM ANY CLAIMS OF INFRINGEMENT, AND YOU ARE RESPONSIBLE
 * FOR OBTAINING ANY RIGHTS YOU MAY REQUIRE FOR YOUR IMPLEMENTATION.
 * XILINX EXPRESSLY DISCLAIMS ANY WARRANTY WHATSOEVER WITH RESPECT TO
 * THE ADEQUACY OF THE IMPLEMENTATION, INCLUDING BUT NOT LIMITED TO
 * ANY WARRANTIES OR REPRESENTATIONS THAT THIS IMPLEMENTATION IS FREE
 * FROM CLAIMS OF INFRINGEMENT, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

/*
 * helloworld.c: simple test application
 */

#include <stdio.h>
#include "platform.h"
#include <xil_io.h>
#include <xintc.h>
#include <xparameters.h>
#include <xil_exception.h>

/************************** Constant Definitions *****************************/

#define FAST_IRQ_EN 1

/*
 * The following constants map to the names of the hardware instances that
 * were created in the EDK XPS system.  They are only defined here such that
 * a user can easily change all the needed device IDs in one place.
 */
#define IRQ_GEN_ID				0
#define IRQ_GEN_INTERRUPT_ID	XPAR_MICROBLAZE_0_INTC_IRQ_GEN_0_IRQ_INTR

#define INTC_DEVICE_ID	XPAR_INTC_0_DEVICE_ID
#define INTC			XIntc
#define INTC_HANDLER	XIntc_InterruptHandler

#define IRQ_PCORE_GEN_BASE  XPAR_IRQ_GEN_0_BASEADDR

// base address of PS Uart1
#define UART_BASE 0xE0001000

// OCM memory used to communicate with CPU0
#define COMM_VAL  (*(volatile u32 *)(0xFFFF0000))

/**************************** Type Definitions *******************************/

/**
 * This typedef contains configuration information for the device driver.
 */
typedef struct {
	u16 DeviceId; /**< Unique ID of device */
	u32 BaseAddress; /**< Base address of the device */
} IrqGen_Config;

/**
 * The driver instance data. The user is required to allocate a
 * variable of this type.
 * A pointer to a variable of this type is then passed to the driver API
 * functions.
 */
typedef struct {
	IrqGen_Config Config; /**< Hardware Configuration */
	u32 IsReady; /**< Device is initialized and ready */
	u32 IsStarted; /**< Device is running */
} XIrqGenIrq;

/***************** Macros (Inline Functions) Definitions *********************/

/************************** Function Prototypes ******************************/

int SetupInterruptSystem();
void XUartChanged_SendByte(u32 BaseAddress, u8 Data);
void outbyte(char c);

#ifdef FAST_IRQ_EN
static void IrqGenIntrHandler(void *CallBackRef) __attribute__((fast_interrupt));
#else
static void IrqGenIntrHandler(void *CallBackRef);
#endif

/************************** Variable Definitions *****************************/

static INTC Intc; /* The Instance of the Interrupt Controller Driver */
static XIrqGenIrq IrqGenInstancePtr; /* The Instance of the IrqGen Driver */

volatile static int irq_count;		// Global for IRQ communication to main()

/****************************************************************************/
/**
 * This function is the main function for the Microblaze in the AMP PS/MB xapp
 * It is responsible for initializing the irq generator device, setting up
 * interrupts, communicating with the PS via OCM, and writing output to the
 * PS UART
 *
 * @param	None.
 *
 * @return
 *		- XST_SUCCESS to indicate success.
 *		- XST_FAILURE to indicate Failure.
 *
 * @note		None.
 *
 *
 *****************************************************************************/
//#define SG_PLBIFACE_COSIM (1)
#include "xparameters.h"
#include "wal.h"
#include "wal_bce_jk.h"

#define SIMD  (8)

#ifdef SG_PLBIFACE_COSIM
#include "sg_plbiface.h"
WAL_REGISTER_WORKER(worker1, BCE_JK_FP32M24, SG_PLBIFACE_ConfigTable, 0, 8, 0);
#else
#include "bce_fp12_1x8_0_axiw.h"

WAL_REGISTER_WORKER(worker1, BCE_JK_FP32M24, BCE_FP12_1X8_0_AXIW_ConfigTable, 0,
		8, 0);
WAL_REGISTER_WORKER(worker2, BCE_JK_FP32M24, BCE_FP12_1X8_0_AXIW_ConfigTable, 1,
		8, 0);
WAL_REGISTER_WORKER(worker3, BCE_JK_FP32M24, BCE_FP12_1X8_0_AXIW_ConfigTable, 2,
		8, 0);
WAL_REGISTER_WORKER(worker4, BCE_JK_FP32M24, BCE_FP12_1X8_0_AXIW_ConfigTable, 3,
		8, 0);
#endif

#include "fill_FA1124P0_program_store.h"
#include "fill_FA1124P1_program_store.h"
#include "fill_FA1101P0_program_store.h"
#include "fill_FA1101P1_program_store.h"

#include "fill_FP1124P0_program_store.h"
#include "fill_FP1124P1_program_store.h"
#include "fill_FP1101P0_program_store.h"
#include "fill_FP1101P1_program_store.h"

#if SG_PLBIFACE_COSIM
#define NN  (10)
#else
#define NN  (250)
#endif

#if SG_PLBIFACE_COSIM
#define MM  (2)
#define NN  (10)
#else
#define MM  (4)
#define NN  (250)
#endif

#define ERR (0.0f)

float af[8][NN], bf[8][NN], zf[8][NN];
float afr[8][NN], bfr[8][NN], zfr[8][NN];

void draw_pb2mb(wal_worker_t *wrk) {
	uint32_t status_led, status_screen;

	wal_pb2mb(wrk, &status_screen);
	while ((status_screen & 0x00000007) != 0) {
		wal_pb2mb(wrk, &status_led);
		while ((status_led & 0x02000000) == 0) {
			wal_pb2mb(wrk, &status_led);
		}
		wal_pb2mb(wrk, &status_screen);
	}
	wal_pb2mb(wrk, &status_led);
	while ((status_led & 0x02000000) == 0) {
		wal_pb2mb(wrk, &status_led);
	}

}

float vprod_mb(unsigned int nn, float * afmb, float * bfmb) {
	unsigned int i, j;
	float zs[4];

	for (i = 0; i < 4; i++) {
		zs[i] = 0;
	}

	j = 0;
	for (i = 0; i < nn; i++) {
		zs[j] += afmb[i] * bfmb[i];
		if (j < 3)
			j = j + 1;
		else
			j = 0;
	}
	zs[0] += zs[1];
	zs[2] += zs[3];
	zs[0] += zs[2];

	return ((float) zs[0]); //vector product
}

void vmac_mb(unsigned int nn, float * afmb, float * bfmb, float * zfmb) {
	unsigned int i;
	for (i = 0; i < nn; i++) {
		zfmb[i] += afmb[i] * bfmb[i];
	}
}

void vmsubac_mb(unsigned int nn, float * afmb, float * bfmb, float * zfmb) {
	unsigned int i;
	for (i = 0; i < nn; i++) {
		zfmb[i] -= afmb[i] * bfmb[i];
	}
}

void bce_fp12_1x8_0_VZ2A(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;

	//====================================================
	if (capab & WAL_BCE_JK_CAP_VZ2A) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VZ2A '%s' .......", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VZ2A;

		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				zf[j][i] = (float) i + (float) j + (float) 0.123456;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &afr[j][0],
					nn);
		}

		k = 0;
		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				if (afr[j][i] != zf[j][i]) {
					xil_printf("\r\n VZ2A Error: afr[%d][%d] = %d", j, i,
							(int) afr[j][i]);
					while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
						;
					k++;
				}
			}
		}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VZ2A not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VB2A(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;

	//====================================================
	if (capab & WAL_BCE_JK_CAP_VB2A) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VB2A '%s' .......", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VB2A;

		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				bf[j][i] = (float) i + (float) j + (float) 0.234567;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &afr[j][0],
					nn);
		}
		k = 0;
		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				if (afr[j][i] != bf[j][i]) {
					xil_printf("\r\n VB2A Error: afr[%d][%d] = %d", j, i,
							(int) afr[j][i]);
					while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
						;
					k++;
				}
			}
		}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VB2A not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VZ2B(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;

	//====================================================
	if (capab & WAL_BCE_JK_CAP_VZ2B) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VZ2B '%s' .......", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VZ2B;

		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				zf[j][i] = (float) i + (float) j + (float) 0.123456;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bfr[j][0],
					nn);
		}

		k = 0;
		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				if (bfr[j][i] != zf[j][i]) {
					xil_printf("\r\n VZ2B Error: bfr[%d][%d] = %d", j, i,
							(int) bfr[j][i]);
					while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
						;
					k++;
				}
			}
		}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VZ2B not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VA2B(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;

	//====================================================
	if (capab & WAL_BCE_JK_CAP_VA2B) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VA2B '%s' .......", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VA2B;

		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				af[j][i] = (float) i + (float) j + (float) 0.234567;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (ah * 0x100), &af[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_B, (ah * 0x100), &bfr[j][0],
					nn);
		}
		k = 0;
		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				if (bfr[j][i] != af[j][i]) {
					xil_printf("\r\n VA2B Error: bfr[%d][%d] = %d", j, i,
							(int) bfr[j][i]);
					while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
						;
					k++;
				}
			}
		}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VA2B not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VADD(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VADD) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VADD '%s' .......", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VADD;
		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				af[j][i] = (float) i + (float) j + (float) 0.123456;
				bf[j][i] = (float) i + (float) j + (float) 0.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[j][i] + bf[j][i];
				err = zfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VADD Error: zfr[%d][%d] = %d   r = %d", j, i,
						(int) zfr[j][i], (int) r);
				while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
				;
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VADD not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VADD_BZ2A(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VADD_BZ2A) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VADD_BZ2A '%s' ..", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VADD_BZ2A;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				bf[j][i] = (float) i + (float) j + (float) 1.23456;
				zf[j][i] = (float) i + (float) j + (float) 1.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &afr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = bf[j][i] + zf[j][i];
				err = afr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VADD_BZ2A Error: afr[%d][%d] = %d   r = %d", j, i,
						(int) afr[j][i], (int) r);
				while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
				;
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VADD_BZ2A not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VADD_AZ2B(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VADD_AZ2B) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VADD_AZ2B '%s' ..", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VADD_AZ2B;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.23456;
				zf[j][i] = (float) i + (float) j + (float) 1.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[j][i] + zf[j][i];
				err = bfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VADD_AZ2B Error: bfr[%d][%d] = %d   r = %d", j, i,
						(int) bfr[j][i], (int) r);
				while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
				;
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VADD_AZ2B not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VSUB(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VSUB) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VSUB '%s' .......", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VSUB;
		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				af[j][i] = (float) i + (float) j + (float) 0.123456;
				bf[j][i] = (float) i + (float) j + (float) 0.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[j][i] - bf[j][i];
				err = zfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VSUB Error: zfr[%d][%d] = %d   r = %d", j, i,
						(int) zfr[j][i], (int) r);
				while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
				;
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VSUB not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VSUB_BZ2A(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VSUB_BZ2A) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VSUB_BZ2A '%s' ..", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VSUB_BZ2A;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				bf[j][i] = (float) i + (float) j + (float) 1.23456;
				zf[j][i] = (float) i + (float) j + (float) 1.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &afr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = bf[j][i] - zf[j][i];
				err = afr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VSUB_BZ2A Error: afr[%d][%d] = %d   r = %d", j, i,
						(int) afr[j][i], (int) r);
				while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
				;
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VSUB_BZ2A not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VSUB_AZ2B(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VSUB_AZ2B) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VSUB_AZ2B '%s' ..", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VSUB_AZ2B;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.23456;
				zf[j][i] = (float) i + (float) j + (float) 1.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[j][i] - zf[j][i];
				err = bfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VSUB_AZ2B Error: bfr[%d][%d] = %d   r = %d", j, i,
						(int) bfr[j][i], (int) r);
				while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
				;
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VSUB_AZ2B not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VMULT(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMULT) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMULT '%s' ......", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMULT;

		for (i = 0; i < nn; i++) {
			for (j = 0; j < SIMD; j++) {
				af[j][i] = (float) i + (float) j + (float) 0.123456;
				bf[j][i] = (float) i + (float) j + (float) 0.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[j][i] * bf[j][i];
				err = zfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VMULT Error: zfr[%d][%d] = %d   r = %d", j, i,
						(int) zfr[j][i], (int) r);
				while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
				;
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMULT not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VMULT_BZ2A(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMULT_BZ2A) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMULT_BZ2A '%s' .", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMULT_BZ2A;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				bf[j][i] = (float) i + (float) j + (float) 1.23456;
				zf[j][i] = (float) i + (float) j + (float) 1.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &afr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = bf[j][i] * zf[j][i];
				err = afr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VMULT_BZ2A Error: afr[%d][%d] = %d   r = %d", j, i,
						(int) afr[j][i], (int) r);
				while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
				;
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMULT_BZ2A not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VMULT_AZ2B(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMULT_AZ2B) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMULT_AZ2B '%s' .", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMULT_AZ2B;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.23456;
				zf[j][i] = (float) i + (float) j + (float) 1.789;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[j][i] * zf[j][i];
				err = bfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VMULT_AZ2B Error: bfr[%d][%d] = %d   r = %d", j, i,
						(int) bfr[j][i], (int) r);
				while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
				;
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMULT_AZ2B not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VPROD(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VPROD) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VPROD '%s' ......", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		//definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 0;
		op = WAL_BCE_JK_VPROD;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.123456;
				bf[j][i] = (float) i + (float) j + (float) 0.16543;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0], 1);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			r = vprod_mb(nn, &af[j][0], &bf[j][0]);
			err = zfr[j][0] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VPROD Error: zfr[%d][0] = %d   r = %d",
					(int) zfr[j][0], (int) r);
			while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
			k++;
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VPROD not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VMAC(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	float err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMAC) {
		if (NN > 10) {
			nn = 10;
		} else {
			nn = NN;
		}
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMAC '%s' .......", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMAC;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.123456;
				bf[j][i] = (float) i + (float) j + (float) 0.16543;
				zf[j][i] = (float) 0.0;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			vmac_mb(nn, &af[j][0], &bf[j][0], &zf[j][0]);
			for (i = 0; i < nn; i++) {
				err = zf[j][i] - zfr[j][i];
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VMAC Error: zf[%d][%d] = %d   zf1r[%d] = %d", j, i,
						(int) zf[j][i], i, (int) zfr[j][i]);
				while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
				;
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMAC not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VMSUBAC(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VMSUBAC) {
		if (NN > 10) {
			nn = 10;
		} else {
			nn = NN;
		}
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMSUBAC '%s' ....", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VMSUBAC;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.123456;
				bf[j][i] = (float) i + (float) j + (float) 0.16543;
				zf[j][i] = (float) 0.0;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			vmsubac_mb(nn, &af[j][0], &bf[j][0], &zf[j][0]);
			for (i = 0; i < nn; i++) {
				err = zf[j][i] - zfr[j][i];
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VMSUBAC Error: zf[%d][%d] = %d   zf1r[%d] = %d", j, i,
						(int) zf[j][i], i, (int) zfr[j][i]);
				while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
				;
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VMSUBAC not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VPROD_S8(wal_worker_t *wrk, unsigned int capab,
		unsigned int ah, unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VPROD_S8) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VPROD_S8 '%s' ...", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 0;
		op = WAL_BCE_JK_VPROD_S8;

		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				af[j][i] = (float) i + (float) j + (float) 1.123456;
				bf[j][i] = (float) i + (float) j + (float) 2.76543;
			}
		}

		for (j = 0; j < SIMD; j++) {
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[j][0], nn);
			wal_mb2dmem(wrk, j, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[j][0], nn);
		}

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0], 1);
		}

		k = 0;
		r = ((vprod_mb(nn, &af[0][0], &bf[0][0])
				+ vprod_mb(nn, &af[1][0], &bf[1][0]))
				+ (vprod_mb(nn, &af[2][0], &bf[2][0])
						+ vprod_mb(nn, &af[3][0], &bf[3][0])))
				+ ((vprod_mb(nn, &af[4][0], &bf[4][0])
						+ vprod_mb(nn, &af[5][0], &bf[5][0]))
						+ (vprod_mb(nn, &af[6][0], &bf[6][0])
								+ vprod_mb(nn, &af[7][0], &bf[7][0])));

		for (j = 0; j < SIMD; j++) {
			err = zfr[j][0] - r;
			if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
			xil_printf("\r\n VPROD_S8 Error: zfr[%d][0] = %d   r = %d", j,
					(int) zfr[j][0], (int) r);
			while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
			k++;
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VPROD_S8 not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

void bce_fp12_1x8_0_VDIV(wal_worker_t *wrk, unsigned int capab, unsigned int ah,
		unsigned int bh, unsigned int zh) {
	float r, err;
	unsigned int i, j, k;
	unsigned int nn, a, as, b, bs, z, zs, inca, incb, incz, op;
	//====================================================
	if (capab & WAL_BCE_JK_CAP_VDIV) {
		nn = NN;
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VDIV '%s' .......", wrk->name);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		// definition of vectors
		a = 0;
		as = 0;
		b = 0;
		bs = 0;
		z = 0;
		zs = 0;
		inca = 1;
		incb = 1;
		incz = 1;
		op = WAL_BCE_JK_VDIV;

		for (i = 0; i < nn; i++) {
			af[0][i] = (float) i + (float) 0.123456;
			bf[0][i] = (float) i + (float) 0.789;
		}

		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_A, (ah * 0x100), &af[0][0], nn);
		wal_mb2dmem(wrk, 0, WAL_BCE_JK_DMEM_B, (bh * 0x100), &bf[0][0], nn);

		wal_bce_jk_create_operation(wrk, WAL_PBID_P0, op, a, as, b, bs, z, zs,
				ah, bh, zh, inca, incb, incz, nn);
		draw_pb2mb(wrk);
		wal_bce_jk_sync_operation(wrk);

		for (j = 0; j < SIMD; j++) {
			wal_dmem2mb(wrk, j, WAL_BCE_JK_DMEM_Z, (zh * 0x100), &zfr[j][0],
					nn);
		}

		k = 0;
		for (j = 0; j < SIMD; j++) {
			for (i = 0; i < nn; i++) {
				r = af[0][i] / bf[0][i];
				err = zfr[j][i] - r;
				if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
				xil_printf("\r\n VDIV Error: zfr[%d][%d] = %d   r = %d", j, i,
						(unsigned int) zfr[j][i], (unsigned int) r);
				while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
				;
				k++;
			}
		}
	}
		if (k == 0) {
			xil_printf(" OK");
		}
	} else {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) VDIV not supported");
	}
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	//====================================================
}

int edkdsp_eval_op(worker) {
	unsigned int i;
	unsigned int ah, bh, zh;
	unsigned int capabilities1, capabilities2, capabilities3, capabilities4 ;
	unsigned int license1, license2, license3, license4;

	if (wal_init_worker(worker1)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		return (-1);
	}

	if (wal_init_worker(worker2)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		return (-1);
	}

	if (wal_init_worker(worker3)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		return (-1);
	}

	if (wal_init_worker(worker4)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		return (-1);
	}

	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Write firmware ...");
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;

	///////////////////////////////////////////////
	// Upgrade firmware
	for (i = 0; i < 3; i++){
		fill_FP1101P0_program_store[i] = fill_FA1101P0_program_store[i];
		fill_FP1101P1_program_store[i] = fill_FA1101P1_program_store[i];
	}
	for (i = EDKDSP_BOOT; i < WAL_FW_FULL_SIZE; i++){
		fill_FP1101P0_program_store[i] = fill_FA1101P0_program_store[i];
		fill_FP1101P1_program_store[i] = fill_FA1101P1_program_store[i];
	}

	///////////////////////////////////////////////
	//Program and initialize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker1, WAL_PBID_P0, fill_FP1101P0_program_store, -1);
	wal_set_firmware(worker1, WAL_PBID_P1, fill_FP1101P1_program_store, -1);
	wal_get_capabilities(worker1, WAL_PBID_P0, &capabilities1);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities1 = %x", capabilities1);
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;

	///////////////////////////////////////////////
	//Program and initialize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker2, WAL_PBID_P0, fill_FP1101P0_program_store, -1);
	wal_set_firmware(worker2, WAL_PBID_P1, fill_FP1101P1_program_store, -1);
	wal_get_capabilities(worker2, WAL_PBID_P0, &capabilities2);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities2 = %x", capabilities2);
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;

	///////////////////////////////////////////////
	//Program and initialize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker3, WAL_PBID_P0, fill_FP1101P0_program_store, -1);
	wal_set_firmware(worker3, WAL_PBID_P1, fill_FP1101P1_program_store, -1);
	wal_get_capabilities(worker3, WAL_PBID_P0, &capabilities3);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities3 = %x", capabilities3);
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;

	///////////////////////////////////////////////
	//Program and initialize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker4, WAL_PBID_P0, fill_FP1101P0_program_store, -1);
	wal_set_firmware(worker4, WAL_PBID_P1, fill_FP1101P1_program_store, -1);
	wal_get_capabilities(worker4, WAL_PBID_P0, &capabilities4);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities4 = %x", capabilities4);
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;

	for (ah = 0; ah < 1; ah++) {
		for (bh = 0; bh < 1; bh++) {
			for (zh = 0; zh < 1; zh++) {
				if (worker == 1)
				{
					///////////////////////////////////////////////
					// Test of the HW evaluation license
					wal_get_license(worker1, WAL_PBID_P0, &license1);
					if (license1) {
							;
						bce_fp12_1x8_0_VZ2A(worker1, capabilities1, ah, bh, zh);
						bce_fp12_1x8_0_VB2A(worker1, capabilities1, ah, bh, zh);
						bce_fp12_1x8_0_VZ2B(worker1, capabilities1, ah, bh, zh);
						bce_fp12_1x8_0_VA2B(worker1, capabilities1, ah, bh, zh);
						bce_fp12_1x8_0_VADD(worker1, capabilities1, ah, bh, zh);
						bce_fp12_1x8_0_VADD_BZ2A(worker1, capabilities1, ah, bh,
								zh);
						bce_fp12_1x8_0_VADD_AZ2B(worker1, capabilities1, ah, bh,
								zh);
						bce_fp12_1x8_0_VSUB(worker1, capabilities1, ah, bh, zh);
						bce_fp12_1x8_0_VSUB_BZ2A(worker1, capabilities1, ah, bh,
								zh);
						bce_fp12_1x8_0_VSUB_AZ2B(worker1, capabilities1, ah, bh,
								zh);
						bce_fp12_1x8_0_VMULT(worker1, capabilities1, ah, bh, zh);
						bce_fp12_1x8_0_VMULT_BZ2A(worker1, capabilities1, ah, bh,
								zh);
						bce_fp12_1x8_0_VMULT_AZ2B(worker1, capabilities1, ah, bh,
								zh);
						bce_fp12_1x8_0_VPROD(worker1, capabilities1, ah, bh, zh);
						bce_fp12_1x8_0_VMAC(worker1, capabilities1, ah, bh, zh);
						bce_fp12_1x8_0_VMSUBAC(worker1, capabilities1, ah, bh, zh);
						bce_fp12_1x8_0_VPROD_S8(worker1, capabilities1, ah, bh, zh);
						bce_fp12_1x8_0_VDIV(worker1, capabilities1, ah, bh, zh);
					} else {
						xil_printf(
								"\r\n\r\n Evaluation license will expire soon. Please contact:");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf(
								"\r\n UTIA AV CR, v.v.i.; Department of Signal Processing");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf("\r\n Pod vodarenskou vezi 4");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf("\r\n 18208, Prague 8, Czech republic");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf("\r\n http://zs.utia.cas.cz \r\n");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
					}
				}
				else if (worker == 2)
				{
					///////////////////////////////////////////////
					// Test of the HW evaluation license
					wal_get_license(worker2, WAL_PBID_P0, &license2);
					if (license2) {
							;
						bce_fp12_1x8_0_VZ2A(worker2, capabilities2, ah, bh, zh);
						bce_fp12_1x8_0_VB2A(worker2, capabilities2, ah, bh, zh);
						bce_fp12_1x8_0_VZ2B(worker2, capabilities2, ah, bh, zh);
						bce_fp12_1x8_0_VA2B(worker2, capabilities2, ah, bh, zh);
						bce_fp12_1x8_0_VADD(worker2, capabilities2, ah, bh, zh);
						bce_fp12_1x8_0_VADD_BZ2A(worker2, capabilities2, ah, bh,
								zh);
						bce_fp12_1x8_0_VADD_AZ2B(worker2, capabilities2, ah, bh,
								zh);
						bce_fp12_1x8_0_VSUB(worker2, capabilities2, ah, bh, zh);
						bce_fp12_1x8_0_VSUB_BZ2A(worker2, capabilities2, ah, bh,
								zh);
						bce_fp12_1x8_0_VSUB_AZ2B(worker2, capabilities2, ah, bh,
								zh);
						bce_fp12_1x8_0_VMULT(worker2, capabilities2, ah, bh, zh);
						bce_fp12_1x8_0_VMULT_BZ2A(worker2, capabilities2, ah, bh,
								zh);
						bce_fp12_1x8_0_VMULT_AZ2B(worker2, capabilities2, ah, bh,
								zh);
						bce_fp12_1x8_0_VPROD(worker2, capabilities2, ah, bh, zh);
						bce_fp12_1x8_0_VMAC(worker2, capabilities2, ah, bh, zh);
						bce_fp12_1x8_0_VMSUBAC(worker2, capabilities2, ah, bh, zh);
						bce_fp12_1x8_0_VPROD_S8(worker2, capabilities2, ah, bh, zh);
						bce_fp12_1x8_0_VDIV(worker2, capabilities2, ah, bh, zh);
					} else {
						xil_printf(
								"\r\n\r\n Evaluation license will expire soon. Please contact:");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf(
								"\r\n UTIA AV CR, v.v.i.; Department of Signal Processing");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf("\r\n Pod vodarenskou vezi 4");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf("\r\n 18208, Prague 8, Czech republic");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf("\r\n http://zs.utia.cas.cz \r\n");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
					}
				}
				else if (worker == 3)
				{
					///////////////////////////////////////////////
					// Test of the HW evaluation license
					wal_get_license(worker3, WAL_PBID_P0, &license3);
					if (license3) {
							;
						bce_fp12_1x8_0_VZ2A(worker3, capabilities3, ah, bh, zh);
						bce_fp12_1x8_0_VB2A(worker3, capabilities3, ah, bh, zh);
						bce_fp12_1x8_0_VZ2B(worker3, capabilities3, ah, bh, zh);
						bce_fp12_1x8_0_VA2B(worker3, capabilities3, ah, bh, zh);
						bce_fp12_1x8_0_VADD(worker3, capabilities3, ah, bh, zh);
						bce_fp12_1x8_0_VADD_BZ2A(worker3, capabilities3, ah, bh,
								zh);
						bce_fp12_1x8_0_VADD_AZ2B(worker3, capabilities3, ah, bh,
								zh);
						bce_fp12_1x8_0_VSUB(worker3, capabilities3, ah, bh, zh);
						bce_fp12_1x8_0_VSUB_BZ2A(worker3, capabilities3, ah, bh,
								zh);
						bce_fp12_1x8_0_VSUB_AZ2B(worker3, capabilities3, ah, bh,
								zh);
						bce_fp12_1x8_0_VMULT(worker3, capabilities3, ah, bh, zh);
						bce_fp12_1x8_0_VMULT_BZ2A(worker3, capabilities3, ah, bh,
								zh);
						bce_fp12_1x8_0_VMULT_AZ2B(worker3, capabilities3, ah, bh,
								zh);
						bce_fp12_1x8_0_VPROD(worker3, capabilities3, ah, bh, zh);
						bce_fp12_1x8_0_VMAC(worker3, capabilities3, ah, bh, zh);
						bce_fp12_1x8_0_VMSUBAC(worker3, capabilities3, ah, bh, zh);
						bce_fp12_1x8_0_VPROD_S8(worker3, capabilities3, ah, bh, zh);
						bce_fp12_1x8_0_VDIV(worker3, capabilities3, ah, bh, zh);
					} else {
						xil_printf(
								"\r\n\r\n Evaluation license will expire soon. Please contact:");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf(
								"\r\n UTIA AV CR, v.v.i.; Department of Signal Processing");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf("\r\n Pod vodarenskou vezi 4");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf("\r\n 18208, Prague 8, Czech republic");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf("\r\n http://zs.utia.cas.cz \r\n");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
					}
				}
				else if (worker == 4)
				{
					///////////////////////////////////////////////
					// Test of the HW evaluation license
					wal_get_license(worker4, WAL_PBID_P0, &license4);
					if (license4) {
							;
						bce_fp12_1x8_0_VZ2A(worker4, capabilities4, ah, bh, zh);
						bce_fp12_1x8_0_VB2A(worker4, capabilities4, ah, bh, zh);
						bce_fp12_1x8_0_VZ2B(worker4, capabilities4, ah, bh, zh);
						bce_fp12_1x8_0_VA2B(worker4, capabilities4, ah, bh, zh);
						bce_fp12_1x8_0_VADD(worker4, capabilities4, ah, bh, zh);
						bce_fp12_1x8_0_VADD_BZ2A(worker4, capabilities4, ah, bh,
								zh);
						bce_fp12_1x8_0_VADD_AZ2B(worker4, capabilities4, ah, bh,
								zh);
						bce_fp12_1x8_0_VSUB(worker4, capabilities4, ah, bh, zh);
						bce_fp12_1x8_0_VSUB_BZ2A(worker4, capabilities4, ah, bh,
								zh);
						bce_fp12_1x8_0_VSUB_AZ2B(worker4, capabilities4, ah, bh,
								zh);
						bce_fp12_1x8_0_VMULT(worker4, capabilities4, ah, bh, zh);
						bce_fp12_1x8_0_VMULT_BZ2A(worker4, capabilities4, ah, bh,
								zh);
						bce_fp12_1x8_0_VMULT_AZ2B(worker4, capabilities4, ah, bh,
								zh);
						bce_fp12_1x8_0_VPROD(worker4, capabilities4, ah, bh, zh);
						bce_fp12_1x8_0_VMAC(worker4, capabilities4, ah, bh, zh);
						bce_fp12_1x8_0_VMSUBAC(worker4, capabilities4, ah, bh, zh);
						bce_fp12_1x8_0_VPROD_S8(worker4, capabilities4, ah, bh, zh);
						bce_fp12_1x8_0_VDIV(worker4, capabilities4, ah, bh, zh);
					} else {
						xil_printf(
								"\r\n\r\n Evaluation license will expire soon. Please contact:");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf(
								"\r\n UTIA AV CR, v.v.i.; Department of Signal Processing");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf("\r\n Pod vodarenskou vezi 4");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf("\r\n 18208, Prague 8, Czech republic");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
						xil_printf("\r\n http://zs.utia.cas.cz \r\n");
						while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
							;
					}
				}
			}
		}
	}



	xil_printf("\r\n");
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	return (0);
}


void draw_line1(void) {
	xil_printf("\r\n MB0 : (HW FP unit   ) Far-end signal ... ");
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
}

void draw_line2(void) {
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) FIR room response ... ");
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
}

void draw_line3(void) {
	xil_printf("\r\n MB0 : (HW FP unit   ) Add near-end signal ... ");
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
}

void draw_line4(void) {
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) LMS Identification ... ");
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
}

void draw_line5(void) {
	xil_printf("\r\n MB0 : (HW FP unit   ) LMS Identification ... ");
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
}

void draw_line6(float *zf_mu_pb2, float *zf_mu_mb2, float *zf_mu_pb3,
		float *zf_mu_mb3) {
	int i, j;
	float err;

	j = 0;
	for (i = 0; i < NN; i++) {
		err = zf_mu_pb2[i] - zf_mu_mb2[i];
		if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
		xil_printf("\r\n ERROR zf_mu_pb2[%d]", i);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08);
		j++;
	}
}
	for (i = 0; i < NN; i++) {
		err = zf_mu_pb3[i] - zf_mu_mb3[i];
		if (((float) err > (float) ERR)|| ((float) err < (float) -ERR)){
		xil_printf("\r\n ERROR zf_mu_pb3[%d]", i);
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08);
		j++;
	}
}
	if (j == 0) {
		xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) OK");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
	}
}

//**************************************************************************
// Precomputed Carrier and Modulation Data and Routines to Build
//**************************************************************************

static float carrier[] = { 0.000000, 0.382683, 0.707107, 0.923880, 1.000000,
		0.923880, 0.707107, 0.382683 };

static float modulation[164] = { 0.000000, 0.019155, 0.038303, 0.057437,
		0.076549, 0.095634, 0.114683, 0.133691, 0.152649, 0.171552, 0.190391,
		0.209161, 0.227854, 0.246463, 0.264982, 0.283403, 0.301721, 0.319927,
		0.338017, 0.355982, 0.373817, 0.391515, 0.409069, 0.426472, 0.443720,
		0.460804, 0.477720, 0.494460, 0.511019, 0.527390, 0.543568, 0.559546,
		0.575319, 0.590880, 0.606225, 0.621348, 0.636242, 0.650903, 0.665326,
		0.679504, 0.693433, 0.707107, 0.720522, 0.733672, 0.746553, 0.759160,
		0.771489, 0.783535, 0.795293, 0.806759, 0.817929, 0.828799, 0.839365,
		0.849623, 0.859570, 0.869200, 0.878512, 0.887502, 0.896166, 0.904501,
		0.912504, 0.920172, 0.927502, 0.934493, 0.941140, 0.947442, 0.953396,
		0.959001, 0.964253, 0.969152, 0.973695, 0.977881, 0.981708, 0.985175,
		0.988280, 0.991023, 0.993402, 0.995417, 0.997066, 0.998349, 0.999266,
		0.999817, 1.000000, 0.999817, 0.999266, 0.998349, 0.997066, 0.995417,
		0.993402, 0.991023, 0.988280, 0.985175, 0.981708, 0.977881, 0.973695,
		0.969152, 0.964253, 0.959001, 0.953396, 0.947442, 0.941140, 0.934493,
		0.927502, 0.920172, 0.912504, 0.904501, 0.896166, 0.887502, 0.878512,
		0.869200, 0.859570, 0.849623, 0.839365, 0.828799, 0.817929, 0.806759,
		0.795293, 0.783535, 0.771489, 0.759160, 0.746553, 0.733672, 0.720522,
		0.707107, 0.693433, 0.679504, 0.665326, 0.650903, 0.636242, 0.621348,
		0.606225, 0.590880, 0.575319, 0.559546, 0.543568, 0.527390, 0.511019,
		0.494460, 0.477720, 0.460804, 0.443720, 0.426472, 0.409069, 0.391515,
		0.373817, 0.355982, 0.338017, 0.319927, 0.301721, 0.283403, 0.264982,
		0.246463, 0.227854, 0.209161, 0.190391, 0.171552, 0.152649, 0.133691,
		0.114683, 0.095634, 0.076549, 0.057437, 0.038303, 0.019155 };
float noise[2048] = { -1.512935e-002, 1.605920e-001, 1.520783e-001,
		-9.885638e-002, 1.448339e-001, 4.183179e-002, 1.006064e-001,
		1.150101e-001, 3.021423e-002, 4.801659e-002, -7.191015e-003,
		5.095307e-002, -7.870364e-002, -1.128330e-001, -8.685936e-003,
		-5.704404e-002, -4.067111e-003, 3.729908e-002, 6.189105e-002,
		-1.242604e-001, 1.213402e-002, -2.165510e-002, -1.090662e-001,
		2.472293e-002, 7.895108e-002, 1.258496e-001, 1.861978e-001,
		-1.311259e-002, 7.459302e-002, 3.662720e-002, 8.961627e-002,
		3.219810e-002, 7.497481e-002, 8.695455e-002, 8.076056e-002,
		1.054493e-001, 5.358889e-002, 4.683577e-002, 6.247039e-002,
		1.008596e-001, 7.768132e-003, -3.841920e-002, -1.487235e-001,
		-1.051001e-001, 2.207749e-002, 1.552397e-001, 1.285224e-001,
		5.948055e-002, -5.499931e-002, 1.386467e-001, -2.411799e-001,
		3.213353e-002, 4.887750e-002, 3.447578e-003, 1.076999e-002,
		3.966768e-002, 6.494028e-002, -4.352047e-002, 4.583340e-002,
		1.923715e-001, 8.396742e-002, -4.841416e-002, 9.967322e-002,
		1.475255e-002, 1.004773e-001, -7.724830e-002, -1.578235e-001,
		-5.072933e-002, -1.120415e-001, -4.310271e-002, -6.318893e-003,
		2.514985e-001, 1.893284e-002, -6.659781e-002, 3.689076e-002,
		-1.792043e-002, 8.588582e-002, 1.575697e-001, 1.788849e-001,
		-4.967988e-002, 9.341810e-002, 8.669558e-002, 5.016632e-002,
		5.547390e-002, -1.749866e-001, -1.093008e-001, -4.904512e-002,
		-1.855940e-001, 4.354080e-003, -5.771978e-002, -1.196005e-001,
		-6.488284e-002, -1.401133e-001, -6.039437e-002, -9.295934e-003,
		6.038213e-002, 1.123783e-001, 4.739432e-002, -1.704699e-002,
		5.725886e-002, 4.826105e-002, 8.242849e-002, 2.175375e-002,
		-7.803563e-002, 3.431812e-003, 5.598812e-003, -1.541841e-001,
		2.845839e-002, 5.011160e-002, 2.745210e-002, -8.732187e-002,
		2.377548e-001, -1.497396e-001, 2.323471e-002, -6.666202e-002,
		-1.295945e-003, -2.146974e-001, 1.182240e-001, 1.755898e-001,
		-4.478790e-003, 6.777355e-002, -1.968680e-002, 1.086940e-001,
		7.859345e-002, -7.067054e-002, -4.877935e-002, 1.962967e-001,
		8.614578e-002, 8.532405e-002, -1.059784e-002, 1.419811e-001,
		2.292566e-001, -2.548352e-001, 8.490306e-002, -1.214876e-002,
		2.806479e-002, 6.329456e-002, 6.398657e-003, 1.372663e-001,
		-5.109641e-002, -1.325050e-001, 1.672334e-002, -2.038296e-001,
		-1.599357e-001, -1.389232e-001, -6.454920e-002, 9.011746e-002,
		-1.085785e-001, -9.279868e-002, 9.126031e-002, 1.000807e-001,
		-1.019413e-001, 1.033819e-001, 6.554568e-002, 7.063790e-003,
		-1.482208e-001, -1.332812e-001, -1.372934e-001, 8.419031e-002,
		-5.521707e-002, 4.743154e-002, -6.006253e-002, -6.981234e-002,
		-8.567208e-002, -1.129933e-001, 7.732878e-002, -2.888152e-002,
		9.929329e-002, -6.234971e-002, 9.341166e-003, -7.761331e-003,
		-1.330190e-001, -3.754032e-002, -8.373496e-002, 7.630299e-002,
		3.456472e-002, -8.708005e-002, -4.037043e-003, 4.187319e-002,
		-1.023138e-001, 2.321449e-002, -1.288538e-002, 7.152153e-004,
		-3.441113e-002, -3.709729e-002, -1.578765e-001, 1.124737e-001,
		-1.058986e-001, 7.079407e-002, -2.125141e-001, -8.483939e-002,
		-2.017400e-001, 1.696296e-001, 7.222870e-002, -1.574728e-001,
		-1.074648e-001, -7.138495e-002, 9.901556e-002, -9.260614e-003,
		7.506190e-002, 4.123854e-002, 2.870316e-002, 1.413980e-002,
		2.358667e-002, -9.502435e-004, 1.101116e-002, 8.523677e-002,
		1.439863e-001, 4.442620e-002, -1.041179e-001, -4.480731e-002,
		4.836275e-002, 4.204903e-002, 9.524143e-002, -9.542688e-003,
		-6.735473e-003, 2.282885e-002, 1.240171e-001, 9.707757e-002,
		7.258455e-002, -6.203109e-002, -1.272029e-001, -8.643329e-002,
		-3.392487e-002, 2.955350e-002, 2.550933e-002, -1.488263e-002,
		6.729968e-002, -3.874310e-002, -1.056907e-001, 1.756436e-001,
		-1.305396e-001, 2.046925e-001, -3.376912e-002, 5.333030e-002,
		-5.302686e-002, -1.371134e-001, 1.331727e-002, 1.739148e-002,
		-2.181676e-002, 6.628352e-002, 1.289604e-002, -1.081658e-001,
		3.432099e-002, -1.388664e-002, 4.725063e-002, -2.710375e-002,
		3.757818e-002, -4.367206e-002, -1.179749e-002, -1.253308e-001,
		6.100506e-002, 1.331786e-001, -3.239430e-002, -6.017594e-002,
		-9.987841e-002, -7.644104e-002, -1.721960e-001, -9.640088e-003,
		-1.717261e-001, 1.331090e-001, 7.913455e-002, -1.031810e-001,
		1.596561e-001, -8.719651e-002, -5.198306e-002, 6.796706e-003,
		-8.421641e-003, -8.398373e-002, -4.101733e-002, 1.127045e-002,
		-1.120706e-002, 9.073892e-002, -9.729237e-002, 1.033882e-001,
		8.746062e-002, 1.246735e-001, -7.169615e-002, -1.641754e-001,
		-1.102792e-001, 9.561807e-002, -1.602030e-002, 9.199493e-002,
		-8.754495e-002, -4.422316e-002, 2.618943e-002, 8.522980e-002,
		2.075334e-001, -7.111431e-002, -2.023378e-002, 1.007169e-001,
		-6.681120e-002, 1.402824e-001, -4.320551e-002, -1.363376e-001,
		1.665748e-001, 5.887676e-002, 1.009507e-001, -4.297216e-002,
		3.535617e-002, -1.404618e-001, -3.926663e-002, -1.573038e-001,
		-1.339960e-001, 4.673708e-002, 6.240065e-002, 1.086896e-001,
		4.372759e-002, 1.615733e-001, 2.037428e-001, -1.539350e-003,
		4.235494e-003, 1.531360e-001, 4.459342e-002, -6.890530e-003,
		2.985320e-002, -2.461699e-001, 1.937775e-001, 8.236857e-002,
		-8.098801e-002, 4.343530e-002, 1.170956e-002, 1.565663e-001,
		5.252823e-002, 1.518584e-001, -6.581046e-002, -5.819079e-002,
		2.349445e-002, -9.877747e-003, -1.352142e-002, -1.886514e-001,
		-1.489007e-001, 1.483196e-002, 9.539140e-002, 2.293194e-001,
		-1.451637e-002, 7.198376e-003, 7.139857e-002, 1.244613e-001,
		4.961082e-002, 5.580260e-002, -1.939929e-001, 2.434568e-001,
		3.469544e-002, -9.755606e-002, 4.654485e-002, -9.990140e-003,
		-2.805177e-002, 4.135892e-002, 2.966719e-002, 1.155610e-001,
		-9.783560e-002, -1.157093e-001, -5.484816e-003, 1.138700e-001,
		7.721884e-002, 7.238738e-003, 1.896143e-001, -1.111481e-001,
		5.088261e-003, -6.326578e-002, -1.107559e-001, -2.472910e-002,
		6.028404e-003, 2.026435e-001, -3.322156e-002, 6.759755e-002,
		-8.062325e-002, 4.917338e-004, 8.782503e-002, -9.020455e-002,
		1.706929e-003, -3.684274e-002, -3.525539e-002, 3.086058e-002,
		3.290373e-002, -7.881613e-002, -9.734378e-002, -2.151434e-003,
		3.382255e-002, 1.619176e-003, -7.305182e-002, 1.547420e-002,
		2.121121e-001, -1.158545e-002, 1.288570e-001, 1.212061e-001,
		-1.642716e-001, -3.912450e-002, 1.472771e-001, -6.158891e-002,
		8.513665e-002, -6.425618e-002, 9.037343e-002, -2.634621e-002,
		-2.977105e-003, 3.457809e-001, -1.278115e-001, 1.008626e-001,
		1.206540e-001, -5.861954e-002, -2.402624e-001, 1.395397e-001,
		9.768790e-003, -1.106131e-001, 1.002685e-002, -1.674544e-001,
		7.961200e-002, 2.983327e-002, 4.741947e-002, 1.186671e-001,
		-2.262329e-001, -1.188224e-001, 9.160826e-002, 1.059931e-001,
		-1.292739e-002, 3.633572e-002, 4.804580e-002, 8.731215e-002,
		6.144812e-002, -8.555391e-003, -3.382918e-002, -3.258958e-002,
		-1.343653e-001, -1.736039e-001, 6.444446e-002, 1.472652e-001,
		-8.172254e-002, -1.709905e-001, -5.373605e-002, 1.676894e-004,
		-1.018391e-001, 1.418273e-002, 4.197790e-002, -1.949549e-003,
		9.257387e-002, 3.110458e-002, 1.714172e-001, -3.266177e-002,
		-2.158845e-001, -1.156727e-001, 5.235669e-002, 2.612613e-002,
		3.638755e-002, -1.871446e-002, 2.095604e-001, 1.012323e-001,
		-6.150905e-003, 1.044376e-001, -8.351823e-002, -3.585825e-002,
		-7.666007e-002, 4.545704e-002, 2.081867e-001, 3.342502e-002,
		-4.736429e-002, 3.167765e-002, -2.326471e-002, -1.991279e-001,
		-9.611713e-002, 2.555546e-003, 2.198292e-001, -8.006530e-002,
		-2.079211e-002, -8.528267e-002, -6.333323e-002, -1.161243e-001,
		1.358732e-001, 2.175170e-002, -1.419274e-001, 6.413305e-002,
		3.779387e-002, -4.632818e-002, -7.004991e-002, -1.112185e-001,
		8.824701e-002, 2.761555e-002, -2.189698e-002, 1.121797e-001,
		-2.454864e-001, 7.253754e-002, -4.873444e-002, 1.078875e-003,
		1.041268e-002, 1.072170e-002, -1.455962e-002, -8.224212e-002,
		1.041743e-001, -6.911848e-002, 1.083196e-001, 6.731415e-003,
		1.629465e-001, -7.690033e-002, -2.230579e-001, -1.445398e-001,
		-8.034277e-002, 5.618464e-002, -5.450922e-002, -1.853971e-001,
		-3.328013e-002, -1.782251e-001, -4.632305e-002, -1.655340e-001,
		9.694117e-004, -7.840585e-002, -3.733361e-002, -7.600487e-002,
		1.205090e-001, 3.586436e-002, 1.600851e-001, -1.829377e-001,
		-2.696564e-002, 1.187360e-002, 1.240663e-001, 7.546863e-002,
		-1.475427e-001, 1.109232e-001, 4.596827e-002, 2.728731e-002,
		-1.210719e-001, -9.764931e-002, -1.900943e-003, 3.943017e-002,
		1.412503e-001, 6.887138e-002, -5.834122e-003, 2.652492e-002,
		-3.111533e-002, -6.454841e-002, -2.492031e-002, -6.203310e-002,
		9.888851e-002, 4.420856e-003, 1.156969e-001, -2.293764e-001,
		4.499559e-002, -6.086655e-002, 2.612171e-002, -8.074493e-002,
		-6.664257e-002, 2.228755e-002, 2.239371e-001, -4.986233e-002,
		5.533740e-002, -3.454122e-002, -9.817964e-003, -2.056243e-001,
		-1.675959e-001, 8.032436e-002, -1.128619e-001, 1.044263e-001,
		6.446590e-002, 9.862799e-002, -1.976740e-004, 1.838409e-001,
		2.487761e-002, 1.675158e-001, -3.264299e-002, -9.427706e-002,
		-3.403721e-002, 6.583962e-002, -5.777266e-002, -6.877171e-003,
		-6.643416e-002, 3.842160e-002, -1.572831e-002, -8.013099e-002,
		6.800638e-003, 4.767997e-002, -3.371187e-002, 2.284081e-002,
		-1.171503e-002, 9.572360e-002, 3.863197e-002, 5.588757e-002,
		9.681516e-002, -1.058776e-002, -1.690467e-001, -1.251018e-001,
		-8.899712e-002, 3.453774e-002, 7.926851e-002, 2.003824e-002,
		-2.301238e-002, -5.378148e-002, 1.806967e-001, -8.698202e-002,
		2.318784e-003, -1.273448e-002, 8.690113e-002, -2.177974e-002,
		4.554794e-002, -1.085516e-001, -1.714390e-001, 6.271299e-002,
		-3.450972e-002, 2.431694e-002, -3.160286e-002, -6.036275e-002,
		1.123041e-001, 1.063461e-001, 1.661678e-001, 7.463504e-002,
		-1.321277e-002, 5.794793e-002, -8.362404e-002, -1.479329e-002,
		-3.581916e-002, -1.291178e-001, 2.230228e-001, -2.483899e-002,
		-8.749405e-002, -1.263973e-002, -6.989218e-002, 6.098031e-002,
		4.067582e-002, -1.059156e-001, -6.591354e-002, 1.278799e-001,
		1.570722e-001, 6.848622e-002, 1.733327e-003, 2.187966e-001,
		-5.967346e-002, -5.229188e-002, 7.147242e-003, 1.211843e-001,
		-1.108971e-001, 5.400530e-002, -2.958509e-002, -6.592618e-002,
		1.107055e-001, -9.050971e-005, -8.329432e-002, 5.012954e-002,
		7.795789e-002, 3.753739e-002, -1.856157e-001, -1.405286e-001,
		-4.387704e-002, -2.490591e-002, -7.594978e-002, 8.896866e-002,
		-2.563709e-001, 9.672681e-002, 1.251374e-002, 6.730522e-002,
		-2.672422e-002, -1.122234e-001, -2.027954e-002, -3.541412e-002,
		2.082114e-001, 1.252031e-002, 7.446110e-002, 2.872571e-002,
		1.366239e-001, 4.795267e-003, -4.732852e-002, -3.973958e-002,
		-1.817257e-001, 4.105216e-002, 4.768039e-002, -1.729966e-001,
		1.842987e-001, 8.163716e-002, 1.276267e-001, 1.203556e-002,
		7.732615e-002, -1.635914e-001, 4.085183e-003, -7.256565e-002,
		-8.836521e-002, -1.671981e-001, -1.148662e-001, -1.465404e-001,
		-2.450605e-001, 1.572467e-002, -5.389813e-002, -1.072477e-001,
		1.052035e-001, 1.754591e-001, -1.942420e-001, 1.275465e-001,
		-4.772312e-002, 3.125333e-002, 4.652829e-002, -6.139021e-002,
		3.755133e-002, 1.467877e-001, 2.778535e-002, 3.027714e-002,
		-5.618191e-002, -5.156325e-002, 2.325198e-001, -2.516519e-002,
		-6.201979e-003, -6.348775e-002, 1.048760e-001, -1.217898e-001,
		4.174259e-002, 5.149475e-002, -1.268244e-001, 5.651085e-002,
		1.424842e-002, -6.529984e-002, 2.105263e-001, -5.348690e-002,
		-4.735327e-002, -1.249557e-001, -1.553029e-001, -8.943445e-002,
		4.797367e-002, -5.053323e-003, -1.031334e-001, -5.727920e-002,
		7.654068e-002, 4.799012e-002, 1.343160e-003, -4.844862e-002,
		-8.812777e-002, -6.440096e-002, -1.519041e-001, 3.989784e-003,
		6.736523e-002, -6.357815e-002, 6.633877e-002, 1.679859e-001,
		1.201841e-001, 1.158318e-003, -1.627698e-001, 4.890301e-002,
		-4.234911e-002, 1.742570e-001, 7.866187e-002, 1.617413e-003,
		-2.429905e-002, 1.912061e-001, -1.023076e-001, -1.483718e-001,
		-9.687140e-002, -9.683958e-002, -1.922858e-001, 9.323715e-002,
		-1.146768e-002, -6.245239e-002, -9.803979e-002, -4.014669e-002,
		-4.133781e-002, -1.781851e-001, -4.935448e-002, -1.128734e-001,
		1.699553e-001, -1.180912e-001, 6.823796e-002, 1.774138e-001,
		-1.239793e-001, 9.531233e-002, 6.825862e-003, 1.057386e-001,
		4.172152e-002, -1.653759e-001, 1.776036e-002, -1.668976e-001,
		1.684953e-001, 1.534000e-001, -2.854543e-002, 7.090729e-002,
		-5.336571e-002, -2.037436e-001, -9.567764e-002, 1.049209e-001,
		-1.305725e-001, -4.967926e-002, -1.894381e-001, -2.654319e-001,
		-1.969377e-001, 7.077286e-002, 5.821147e-002, 6.487980e-002,
		-1.096836e-001, -7.279304e-002, 1.796639e-001, -1.023761e-001,
		1.927409e-003, -3.014928e-003, 1.922864e-001, -4.627270e-002,
		3.768084e-002, 6.990036e-002, 1.303918e-001, 3.562730e-002,
		-1.821564e-001, -7.282101e-002, -2.368763e-002, 1.936340e-001,
		-3.153897e-002, 4.212645e-002, -1.223097e-001, 6.236014e-002,
		8.788844e-002, -4.605978e-002, 9.663009e-003, 5.128588e-002,
		-1.038006e-001, -7.969621e-002, -5.926868e-002, -1.997443e-001,
		-1.904239e-001, 8.584392e-002, 3.134282e-002, 3.143744e-002,
		-6.773732e-002, 2.182502e-001, -7.835399e-002, 1.087246e-001,
		-9.121218e-003, 1.289929e-001, -6.578862e-002, -2.202352e-001,
		-1.799001e-002, 6.473715e-002, 5.983033e-002, -1.213228e-001,
		-2.948079e-002, 1.405232e-001, 9.627492e-002, -2.072833e-001,
		8.430296e-003, 8.403884e-002, -1.146883e-001, 2.246654e-001,
		6.288916e-002, -6.372585e-002, -3.065090e-002, -1.750225e-002,
		-1.709303e-002, 1.730950e-001, -6.413397e-002, 1.654875e-002,
		4.647170e-002, -1.057078e-001, 2.205118e-002, -4.958233e-002,
		2.953507e-002, 3.577265e-002, -1.547477e-001, -1.211758e-001,
		2.183494e-001, 6.267396e-002, 1.905387e-002, -9.725229e-002,
		-5.550565e-002, 1.036043e-001, -3.913699e-002, 6.968668e-002,
		-2.094282e-002, -4.435642e-002, 9.226627e-002, -1.052581e-001,
		-7.405336e-002, 1.355731e-001, -1.420500e-001, -1.451030e-001,
		-9.701603e-002, -1.362952e-001, 7.001652e-002, 4.142920e-002,
		2.886665e-002, 3.577796e-002, 5.911848e-002, -1.636126e-001,
		-4.767447e-002, 1.332040e-001, 9.652254e-003, -2.011265e-002,
		8.303606e-002, -7.675172e-002, 1.706441e-001, 1.370983e-001,
		-8.777239e-003, 1.541198e-001, 6.068849e-002, -2.910945e-002,
		6.400113e-002, -9.172997e-002, 4.899321e-002, 1.773191e-001,
		-1.888944e-002, -2.658384e-002, -4.488513e-002, 6.863835e-002,
		-1.550447e-001, 1.105550e-002, -2.313732e-002, 3.165922e-002,
		1.011772e-002, -1.161812e-001, -1.211324e-001, -5.083325e-002,
		3.390744e-002, 6.495786e-002, 5.382177e-002, 7.443258e-002,
		1.063697e-001, 1.991975e-001, -6.856962e-002, 9.118036e-002,
		1.515047e-001, 7.234579e-002, 4.594182e-002, 1.454034e-002,
		1.331813e-001, 8.679154e-002, -1.782326e-001, 2.775486e-003,
		9.464527e-003, -5.453172e-002, 8.639118e-002, 1.877258e-002,
		-9.807510e-002, -8.919423e-002, -1.876155e-003, -9.644309e-002,
		-1.966719e-003, -5.050808e-002, 1.723296e-002, -5.884545e-002,
		1.155272e-002, -1.091670e-001, 1.480500e-001, 7.625117e-002,
		1.643130e-001, 5.847582e-002, 3.076149e-002, -3.454831e-002,
		-2.623415e-002, 8.013155e-002, -8.139291e-003, 4.586055e-002,
		-6.359394e-003, 1.576338e-001, -4.499709e-003, -9.316956e-002,
		-2.154474e-001, 7.490692e-002, -2.714303e-002, 6.136638e-002,
		-2.118198e-001, -2.818197e-001, 4.960313e-002, -1.248117e-001,
		4.427065e-002, -1.754897e-001, -3.638936e-002, 4.546722e-002,
		-1.315457e-001, 2.399889e-001, -8.079468e-002, 2.354161e-001,
		-3.274432e-001, 1.886769e-001, 2.888540e-002, -2.654451e-003,
		1.181087e-001, 1.389161e-001, -4.890708e-002, -9.442853e-002,
		-8.432747e-002, 1.829187e-002, -1.107724e-001, -6.605162e-002,
		8.140118e-002, -1.252579e-001, -8.759557e-002, -1.637708e-001,
		-3.269100e-002, -2.069799e-002, 2.832638e-002, -2.720658e-002,
		9.485157e-002, 9.423516e-002, -1.035359e-001, 3.447019e-002,
		-1.547397e-001, -1.408458e-002, -1.028796e-002, -2.337694e-002,
		2.853127e-002, -2.975098e-002, 7.842097e-002, -5.255088e-003,
		-8.028004e-002, -1.171770e-002, 2.417443e-002, 7.724027e-002,
		-5.531980e-002, -1.468391e-001, 1.388109e-001, -1.686282e-001,
		8.412121e-002, -1.029940e-001, 5.673215e-002, -1.524982e-001,
		1.464870e-003, -1.160983e-001, -3.271170e-002, 7.105727e-003,
		-1.389649e-002, -5.637241e-002, 1.089510e-001, 9.720042e-003,
		2.053869e-002, 4.780010e-002, -1.837375e-001, 2.186280e-002,
		5.979271e-003, 9.625826e-002, 4.526099e-002, -1.250708e-001,
		-1.280908e-002, -4.205582e-002, 5.798374e-002, -3.675274e-002,
		-9.214551e-003, -6.893582e-002, 6.114951e-003, 1.938831e-002,
		6.712769e-002, -4.833568e-003, -5.288057e-002, 8.899348e-002,
		1.781467e-001, 3.605552e-002, -1.018310e-002, 1.336085e-001,
		-7.215312e-002, -8.404393e-002, -2.644629e-002, -2.095192e-001,
		3.830816e-002, 2.555359e-002, -1.451139e-001, -3.794591e-002,
		8.053255e-002, -3.325600e-002, 7.032760e-002, -7.024158e-002,
		3.034949e-002, 7.810010e-003, -1.987511e-002, 3.442618e-002,
		-4.585651e-003, -1.445805e-001, 3.265351e-002, 1.441636e-002,
		-2.301666e-003, 1.547955e-002, 7.894479e-002, -1.631224e-001,
		1.388406e-001, 5.500363e-002, -6.982518e-002, 1.905438e-001,
		-1.661441e-002, -3.477523e-002, 1.321543e-002, -1.108715e-001,
		1.400005e-002, 1.454254e-001, -7.827418e-002, -1.547243e-001,
		-1.559616e-001, -4.321840e-002, 2.025404e-002, -1.546181e-001,
		9.787496e-002, -1.923811e-001, 1.768838e-001, -1.887981e-003,
		-3.103282e-001, 4.025801e-004, -2.527913e-003, 3.277504e-002,
		-1.654906e-001, 3.298536e-002, -1.305669e-001, 1.320256e-001,
		3.553895e-002, 5.125850e-003, 8.140313e-003, -1.546900e-001,
		-1.371615e-001, 1.077480e-001, 6.744453e-003, 7.936568e-002,
		-1.536149e-001, -6.392396e-003, -1.540439e-002, -1.306744e-001,
		-8.824134e-002, 1.770353e-002, 6.864542e-002, -1.999832e-001,
		8.108039e-002, -3.604017e-002, -4.873388e-002, 2.912256e-002,
		-3.619241e-002, 3.349928e-002, -3.516383e-002, -2.979718e-002,
		9.714268e-003, 9.236150e-003, 2.167458e-002, -5.614049e-002,
		-6.574846e-002, 4.819675e-002, 1.153299e-002, 6.656925e-002,
		7.881821e-002, -8.539164e-002, -1.316779e-001, 1.715520e-001,
		-2.461868e-002, -2.055526e-001, 2.883635e-002, 1.454652e-002,
		-1.223656e-001, -1.748305e-001, 9.574008e-003, -9.812299e-002,
		1.414800e-001, -3.836827e-002, 5.169672e-003, 7.801077e-002,
		-5.547378e-002, -4.391563e-002, 4.056676e-002, -2.188378e-001,
		7.587856e-003, 1.757552e-004, 1.156922e-001, -4.926510e-002,
		-1.391920e-002, -3.599839e-002, -2.197701e-001, 2.200223e-002,
		-6.030128e-002, 8.420899e-002, -1.623471e-001, 1.823959e-003,
		4.361173e-002, -1.212326e-001, -4.167705e-002, 1.320428e-001,
		1.000802e-001, -1.000189e-001, -1.608217e-002, -4.059149e-002,
		9.802140e-002, 7.056883e-002, 6.068268e-002, 8.384737e-002,
		1.424350e-001, 1.350422e-001, 8.713663e-002, 1.812366e-001,
		-3.222020e-002, 2.821260e-002, 1.386544e-001, -7.456111e-002,
		-5.658732e-002, -6.671686e-002, -6.007751e-002, -2.755486e-003,
		-1.229372e-001, -1.107309e-001, 1.601193e-001, 2.806463e-002,
		1.592855e-002, 1.479270e-001, -4.662869e-002, -1.906187e-001,
		-1.950462e-001, -1.018879e-001, 1.859645e-001, -4.356916e-002,
		1.086609e-001, -1.124188e-001, 6.618925e-002, -5.878157e-002,
		-3.807713e-002, -1.686454e-001, 1.635388e-002, -1.217279e-002,
		8.647615e-002, 9.782932e-002, -1.765324e-001, 1.805003e-001,
		-3.233551e-002, 1.226165e-002, -1.140330e-002, 4.337288e-003,
		-5.434265e-002, 2.023365e-002, 1.376685e-001, 2.816202e-002,
		-2.878637e-002, -1.021741e-002, 1.630688e-001, -3.722731e-002,
		1.523474e-003, 1.148258e-001, 6.402079e-002, 1.519958e-001,
		-1.688271e-001, 5.265268e-002, 8.825369e-002, -9.666648e-003,
		-5.625749e-002, -1.227841e-001, -9.786122e-003, 5.406733e-002,
		7.822848e-002, 7.348991e-002, -1.283534e-001, 6.531902e-002,
		-7.456874e-003, 4.558546e-002, -2.153460e-002, -1.286185e-001,
		3.934734e-002, 2.337353e-001, -1.125450e-002, 2.118736e-001,
		-1.325260e-001, 7.100063e-002, -1.229934e-002, 8.945950e-002,
		3.788335e-002, -8.658482e-002, -4.469965e-002, -5.127134e-002,
		-1.044941e-001, 1.089992e-001, -1.570483e-001, 1.158852e-002,
		-3.884979e-002, -2.095914e-002, -1.980484e-001, 3.947592e-002,
		2.697247e-002, -7.665533e-002, 2.281402e-002, -5.259970e-002,
		4.866016e-002, -1.543343e-002, -1.297080e-001, 3.913506e-002,
		5.733685e-002, -7.914525e-002, 4.911802e-002, -5.703591e-003,
		6.660764e-002, 7.525786e-003, 4.113387e-002, 7.230227e-002,
		1.565620e-001, 7.186354e-002, -1.126188e-002, 1.433855e-001,
		-2.182399e-002, 1.346352e-001, -2.219294e-001, 1.590423e-001,
		-1.160076e-002, -1.495825e-001, -7.014844e-002, -1.666122e-002,
		1.163488e-002, -7.144692e-002, 7.572584e-002, -7.651816e-003,
		-6.739261e-002, 1.495024e-002, -1.024197e-001, -1.696819e-001,
		8.339974e-002, -1.643469e-001, 1.503940e-002, 8.698907e-002,
		5.801002e-002, -1.072053e-001, -7.937895e-002, -1.677380e-003,
		9.315757e-003, -2.173799e-002, 1.076620e-001, -1.740768e-001,
		-1.669665e-001, 6.170112e-003, -2.000957e-001, 9.230584e-002,
		2.691863e-002, 6.515211e-002, 6.033778e-002, -1.285528e-001,
		-1.977174e-003, -7.221151e-002, 8.417177e-002, -1.415711e-001,
		-5.565186e-002, 2.165130e-001, -4.178240e-002, 3.136049e-002,
		-1.774360e-001, 2.024946e-002, -1.357653e-001, 2.670649e-002,
		5.455846e-002, 5.101581e-002, -6.383044e-002, 8.579996e-002,
		-2.748414e-002, -4.349620e-002, 6.262232e-002, 1.631553e-001,
		-1.075274e-001, 1.616270e-002, -8.108836e-002, -1.497540e-002,
		-8.596465e-002, -1.824658e-001, -1.215749e-001, 6.031828e-002,
		-2.461807e-001, -8.080565e-002, -3.635325e-002, 1.717144e-003,
		-1.190400e-001, 7.386153e-002, 1.149804e-001, -8.017487e-002,
		-2.349617e-002, 7.747343e-002, 3.530980e-003, -4.510095e-002,
		8.582206e-002, -6.212412e-002, 3.374944e-003, -1.649191e-001,
		-1.835930e-001, -7.867532e-002, -3.398702e-002, 3.275874e-002,
		9.652064e-002, -5.909570e-002, -5.585059e-002, -1.455967e-002,
		9.031378e-002, -3.715466e-002, 5.205500e-002, 5.187935e-002,
		-1.461675e-002, -5.134041e-003, -4.959694e-002, -1.970622e-001,
		-2.030950e-001, 1.835251e-001, 5.561414e-002, 4.822727e-002,
		-6.555311e-003, -9.278608e-002, 9.126019e-002, 2.361705e-002,
		-1.079248e-001, 9.723300e-002, -1.883903e-002, 7.598992e-003,
		-2.892011e-002, -1.591081e-001, 3.911707e-002, 1.219884e-001,
		4.250278e-002, 1.191176e-001, 4.079558e-002, -2.017206e-001,
		-3.942599e-002, -2.598188e-003, 7.964117e-002, 1.030220e-001,
		1.001097e-001, -8.828583e-002, 7.758385e-002, 2.398550e-002,
		-1.802679e-002, 3.375405e-003, -5.956711e-002, -1.762663e-002,
		-3.874890e-002, 4.030195e-002, -2.389136e-002, 3.694533e-002,
		8.578622e-002, -1.770887e-004, -1.208857e-001, -2.093482e-002,
		8.919757e-002, -2.797523e-002, 1.354476e-001, 3.906688e-002,
		-6.759076e-002, -1.376575e-002, 8.299736e-003, 5.730432e-002,
		-6.025322e-002, 1.762568e-002, 1.256689e-001, 2.724408e-002,
		1.269177e-002, -7.914994e-002, 3.069470e-002, 1.280464e-002,
		-7.636005e-002, 4.143205e-002, 1.720296e-001, -2.723242e-002,
		-5.591793e-002, 4.868914e-002, 5.188295e-002, -7.880564e-002,
		4.115139e-002, -1.096157e-001, 4.679474e-002, 2.122196e-002,
		-4.782500e-003, -9.689707e-002, -3.248197e-002, 4.313794e-002,
		1.503020e-001, 8.573984e-002, -3.318786e-002, -3.393008e-002,
		-5.090284e-002, 7.595398e-002, 1.249754e-002, 9.388191e-002,
		1.539921e-001, 4.244940e-002, 8.863621e-002, 1.060310e-002,
		-1.383509e-001, -1.495521e-002, -1.002942e-001, 1.285988e-005,
		5.353552e-002, -1.792502e-002, 1.679017e-001, -7.875592e-002,
		1.674619e-001, -3.648851e-002, -4.381172e-002, 6.191947e-002,
		-4.203733e-003, 2.112186e-002, -2.831981e-002, 6.615102e-002,
		2.345346e-001, -9.503119e-002, -6.722940e-003, 1.184180e-001,
		3.442647e-002, 1.167372e-001, 4.669730e-002, 2.586171e-002,
		-5.624975e-002, 1.040616e-002, 7.935944e-002, -2.530762e-002,
		1.460006e-001, 5.314362e-002, -1.605649e-001, 6.377339e-002,
		2.513698e-001, 1.066928e-001, 1.202117e-001, -2.241347e-001,
		5.954167e-002, 1.094078e-001, 7.723265e-002, 9.391027e-002,
		-1.319804e-001, -2.776065e-002, -7.134999e-002, -5.640659e-002,
		-7.876812e-002, 5.997681e-003, -7.513230e-004, 3.621317e-002,
		3.352268e-002, -5.666980e-002, 6.869310e-002, -7.512783e-002,
		1.645269e-001, 4.381144e-002, -1.281311e-001, 3.177822e-002,
		-5.257302e-002, 3.788113e-002, 4.966782e-002, 1.880541e-001,
		-1.381546e-003, 2.857419e-002, -6.687676e-002, 1.260557e-001,
		-2.461772e-002, 2.732853e-002, 7.780751e-002, 9.277185e-002,
		-9.348346e-002, -5.947056e-003, 2.783114e-005, 2.031622e-001,
		2.358283e-002, -1.004419e-002, 1.014563e-001, -8.533812e-002,
		-1.141566e-001, 5.207931e-002, -1.565625e-001, -7.679508e-002,
		5.326280e-002, 8.409609e-002, -7.071308e-002, -1.675109e-002,
		1.426684e-001, 9.558915e-002, 7.463463e-002, -1.269490e-002,
		1.169809e-001, 7.988180e-002, 6.960241e-002, -1.391628e-001,
		-1.053005e-002, -1.030788e-001, -4.438709e-002, -4.451442e-002,
		8.404055e-002, 6.891639e-003, 8.817510e-002, 2.032464e-002,
		1.223892e-002, -2.406202e-001, -7.229478e-002, 2.065439e-001,
		-5.822152e-003, -7.114263e-002, -4.868067e-002, -1.979630e-001,
		5.312935e-002, -8.896586e-002, -1.411737e-002, 1.119519e-001,
		1.484401e-001, -1.812830e-001, 1.627336e-001, 9.839473e-002,
		-1.210783e-001, -2.786083e-001, 1.622695e-002, -1.380775e-001,
		5.236799e-002, -1.539069e-001, -1.009602e-001, 7.217407e-003,
		-6.636534e-002, -9.253722e-003, -2.838168e-003, 5.257662e-002,
		2.765117e-001, 2.508014e-002, 5.589665e-002, -3.104247e-001,
		1.457015e-001, -1.559400e-001, -1.381054e-002, -7.386060e-002,
		-8.922578e-002, 1.232483e-001, 4.871324e-002, 9.116025e-002,
		-6.343258e-002, 1.655855e-001, -1.632597e-001, -4.394602e-002,
		-5.446680e-002, -7.854740e-002, -8.719685e-002, 1.898743e-001,
		4.552949e-002, 1.019266e-001, 7.945759e-004, 7.778697e-003,
		1.509630e-001, -4.822894e-002, -9.752267e-003, -4.761344e-002,
		-5.074150e-002, -8.444756e-003, -4.525041e-002, -1.557473e-001,
		1.136780e-001, 2.347301e-002, -7.493460e-002, 1.292267e-001,
		3.855312e-002, -1.475041e-001, -3.682293e-002, 2.774169e-002,
		9.499531e-002, -5.770277e-002, 3.384867e-002, -1.463611e-001,
		-1.785155e-002, 4.465764e-002, -1.634848e-002, -5.869329e-003,
		-1.403824e-001, 2.208468e-001, 1.597189e-001, -1.057931e-001,
		2.698044e-003, -2.122186e-002, 4.190475e-002, 3.799389e-002,
		4.852817e-002, -7.728606e-002, -1.421777e-001, 4.299552e-002,
		3.553965e-002, -7.629609e-003, 1.549433e-001, 7.481229e-002,
		-1.282157e-001, 1.099957e-001, 8.786620e-002, 1.509266e-001,
		-1.806466e-002, 4.004853e-002, -6.051046e-002, 1.184209e-001,
		3.720267e-002, -7.495280e-002, -3.231599e-002, 2.492628e-002,
		6.940183e-002, -5.612231e-002, 5.281101e-002, -1.829481e-002,
		8.860412e-002, 7.587333e-002, 6.609503e-002, 2.311125e-002,
		-3.563544e-002, -1.140895e-001, -7.179728e-002, -1.212890e-001,
		1.253616e-001, -9.835618e-002, -1.352701e-002, 1.385026e-001,
		7.434025e-002, -1.277306e-001, -1.391361e-001, -1.173341e-001,
		1.058342e-002, 1.212699e-003, -9.415624e-002, -9.914381e-002,
		9.213639e-002, -5.125736e-002, -4.423965e-002, 6.641684e-002,
		-2.718446e-003, -1.761518e-001, -1.009356e-001, 1.475689e-001,
		-1.953884e-003, 1.205907e-001, 6.572386e-002, -2.203989e-001,
		-7.292877e-002, 6.945417e-002, -1.021507e-001, 9.992775e-002,
		1.068202e-001, 5.227565e-002, -8.720742e-002, 1.922281e-001,
		-5.521654e-002, 7.271774e-002, -1.383300e-001, -2.323997e-001,
		1.306795e-001, 5.739305e-002, 3.487380e-002, 1.911714e-001,
		-1.785205e-002, -1.788968e-001, 1.182137e-003, -3.342147e-004,
		-8.552047e-002, 4.098997e-002, 4.505608e-002, -2.156027e-001,
		9.955325e-002, 2.296740e-002, 6.054021e-003, -3.226716e-002,
		1.162721e-001, 2.717723e-002, 5.130506e-002, -1.174087e-001,
		3.020769e-002, -1.911003e-001, 7.913057e-002, 1.846727e-001,
		1.108306e-001, 2.444271e-001, 1.071559e-002, 1.584949e-001,
		-2.317036e-002, 6.796204e-002, 5.703319e-002, -6.202675e-002,
		3.584825e-002, -1.484990e-002, 5.650946e-002, 1.008545e-001,
		-7.302950e-003, -7.063147e-002, 1.014547e-002, 1.114270e-001,
		-2.208894e-002, -5.432460e-002, 6.344600e-002, -9.448060e-002,
		-2.161437e-002, -1.589103e-001, -8.258050e-003, -3.583732e-002,
		-3.898170e-002, -5.229225e-002, 2.247539e-001, -2.163657e-002,
		1.019622e-001, -3.845773e-002, -1.895927e-002, -6.835667e-002,
		7.344773e-002, 6.204517e-002, 1.362100e-001, 3.844720e-002,
		-1.117235e-002, -7.341928e-002, -3.541666e-002, 1.246611e-001,
		1.419459e-001, 5.923524e-002, 3.206318e-002, 7.321807e-002,
		3.963718e-002, -9.832225e-002, 1.832489e-003, 2.365054e-002,
		9.994140e-002, -6.564426e-002, -1.112321e-002, 1.053877e-001,
		2.335270e-002, -8.873498e-002, -1.200777e-001, -1.188772e-002,
		4.128512e-002, 1.143953e-001, -1.646164e-001, 1.273458e-002,
		-3.221015e-002, 5.318141e-002, -1.737333e-001, 3.903702e-002,
		-3.287485e-002, 4.831002e-002, -1.824607e-001, -2.755357e-002,
		8.341705e-002, -8.626175e-002, -1.197688e-002, 2.118683e-001,
		-3.993810e-002, -2.489574e-002, -5.602363e-002, -1.050155e-002,
		-5.984508e-002, 2.888196e-001, -4.409522e-002, -7.641882e-002,
		-2.516546e-002, 5.246422e-002, -6.731667e-002, 9.498288e-002,
		1.300242e-001, -1.224405e-001, -2.287175e-002, -2.060133e-002,
		7.631891e-002, 3.428457e-002, 1.593018e-001, 1.172195e-001,
		-1.137067e-001, 3.119608e-002, -1.319038e-001, -6.693830e-002,
		-7.006820e-002, -1.540402e-001, 2.284775e-002, -6.089142e-002,
		9.232646e-002, 1.851211e-001, -5.332552e-002, -5.386830e-002,
		1.696534e-001, -1.766901e-002, 8.083389e-002, 8.094982e-003,
		-6.007857e-004, 1.948921e-001, 2.286811e-002, 1.350365e-002,
		-7.255981e-002, 6.177795e-002, 3.144927e-002, 1.124074e-001,
		1.129101e-001, -4.338881e-002, 8.256694e-002, 8.830051e-002,
		1.463594e-002, 1.184632e-001, -3.263045e-002, -1.309086e-001,
		2.292458e-002, 6.975098e-002, -1.168742e-001, 1.405339e-001,
		1.416784e-001, -1.045722e-001, -1.004525e-001, -7.383961e-002,
		-9.444941e-002, -1.354067e-002, -4.226265e-002, 1.597863e-001,
		-3.810561e-002, 2.348135e-002, -3.093396e-002, 3.409296e-002,
		5.485910e-002, -1.015966e-001, -1.633518e-001, -1.810352e-001,
		-3.490386e-003, 6.757630e-002, -8.908653e-002, -1.891336e-001,
		2.217511e-001, -9.381396e-002, -1.543567e-001, 1.596157e-002,
		-3.688379e-002, -1.012161e-001, 1.133981e-002, 8.849829e-002,
		-5.823166e-002, -7.175522e-002, -6.732523e-002, -3.196913e-002,
		1.606507e-001, 1.061297e-001, 3.005487e-002, 3.511264e-002,
		9.522158e-002, -6.328650e-002, -8.587287e-002, 1.746108e-001,
		-5.561143e-002, -2.426126e-003, 9.170070e-002, -5.014795e-002,
		-2.512873e-002, 1.672802e-001, -1.364408e-001, -3.350653e-002,
		1.294600e-001, 4.811414e-002, -2.520076e-002, 1.011174e-001,
		2.788173e-002, 2.611424e-001, 2.878828e-002, 1.095563e-001,
		2.689213e-002, -8.627340e-002, -1.101806e-001, 3.695743e-002,
		-1.594510e-001, -8.761134e-002, -5.136123e-002, -6.898095e-002,
		8.315252e-002, -1.422392e-001, 1.129109e-001, -1.311292e-001,
		-4.957585e-002, -7.038212e-003, -2.966565e-002, -1.811595e-001,
		1.303373e-001, -2.289353e-002, 7.878915e-002, 3.683245e-002,
		-5.414223e-002, -7.190883e-002, 3.483624e-002, -5.004479e-002,
		7.244483e-002, 5.774340e-002, -1.331503e-001, -1.965991e-001,
		7.047579e-002, -2.063345e-002, 3.787218e-002, -3.179720e-002,
		-6.077619e-002, -1.195338e-001, -7.233887e-002, -6.952789e-003,
		1.103892e-001, 1.473177e-001, -9.305281e-002, -4.713448e-002,
		-1.454644e-001, 5.985343e-002, -3.518646e-002, 1.151063e-001,
		3.983975e-002, 3.259936e-002, -2.285677e-002, -1.272264e-001,
		1.590619e-001, 2.815378e-002, 1.480477e-001, -7.365596e-002,
		6.776419e-002, 4.989215e-002, -4.277870e-002, 2.029432e-001,
		-1.148043e-001, 1.057165e-001, 4.253847e-002, -5.658972e-002,
		-1.456260e-001, 8.427171e-002, -2.001301e-001, 2.617752e-002,
		-2.926468e-003, 8.052639e-002, -1.059674e-003, -1.059162e-001,
		1.200993e-002, -9.803516e-002, 1.360588e-001, -1.431816e-002,
		-8.084196e-002, 3.175605e-002, 7.238663e-005, -1.173088e-001,
		7.902257e-002, 1.750993e-001, 4.496133e-002, 1.334776e-001,
		-5.651807e-002, -1.508841e-001, 8.503386e-002, -7.497464e-002,
		8.262252e-002, -5.800566e-003, 1.998597e-002, 6.598500e-002,
		-6.346630e-002, 8.843347e-002, 2.404214e-001, 1.688466e-002,
		-1.764919e-002, -4.681483e-002, -9.252561e-002, -2.670338e-001,
		1.038510e-002, 6.511113e-002, -2.604112e-003, 3.519549e-002,
		-1.837625e-001, -1.563058e-001, 3.863137e-002, -2.045035e-001,
		1.164373e-001, 6.523418e-002, -6.033097e-003, -1.210180e-001,
		2.102077e-002, -8.154188e-003, -1.456025e-002, -2.899325e-002,
		-3.809719e-002, 5.658870e-002, -4.482665e-002, 6.556652e-002,
		6.015143e-002, 1.100782e-001, 4.878378e-002, 1.769782e-001,
		-2.638049e-003, 9.562256e-002, -1.359784e-001, -9.690314e-003,
		-1.536979e-001, -7.565365e-003, 1.966469e-002, 4.096871e-002,
		-1.242372e-002, 2.198381e-001, 1.126166e-003, 5.787968e-002,
		1.026153e-001, 1.474889e-001, -1.149343e-001, -6.271924e-002,
		-1.263974e-001 };

float ufd[(8 * MM * NN + 8 * NN)]; // system: input measurements
float zfd[(8 * MM * NN + 8 * NN)]; // system: output measurements

float afmb[8 * NN]; // system coef
float bfmb[8 * NN]; // regressor
float zfmb2[NN], zfmb3[NN]; // FIR filter output (MicroBlaze)

float mu[1]; // LMS weight

float af_mu_mb[8 * NN]; // LMS adaptive filter (MicroBlaze)
float bf_mu_mb[8 * NN]; // Regressor (MicroBlaze)
float zf_mu_mb2[NN]; // LMS filter output (MicroBlaze)
float zf_mu_mb3[NN]; // LMS filter output (MicroBlaze)

float af_mu_pb[8 * NN]; // LMS adaptive filter (PicoBlaze)
float bf_mu_pb[8 * NN]; // Regressor (PicoBlaze)
float zf_mu_pb2[NN]; // LMS filter output (PicoBlaze)
float zf_mu_pb3[NN]; // LMS filter output (PicoBlaze)

#if LINUX
void gettimeofday_toc(struct timeval *tmv1, struct timeval *tmv2, unsigned int *usec_toc) {
	tmv2->tv_sec-=tmv1->tv_sec;
	tmv2->tv_usec-=tmv1->tv_usec;
	if (tmv2->tv_usec<0) {
		tmv2->tv_sec--;
		tmv2->tv_usec+=1000000;
	}
	*usec_toc=tmv2->tv_sec*1000000+tmv2->tv_usec;
}
#endif

float get_carrier(int i) {
	if (i % 16 < 8) {
		return (carrier[i % 8]);
	} else {
		return (-(carrier[i % 8]));
	}
}

float get_modulation(int i) {
	if (i % 328 < 164) {
		return (modulation[i % 164]);
	} else {
		return (-(modulation[i % 164]));
	}
}

float get_noise(int i) {
	return (noise[i % 2048]);
}

float get_noise2(int i) {
	return (noise[2047 - (i % 2047)]);
}

//////////////////////////////////////////////////////////////////////////
// DSP_PLATFORM (c) UTIA AV CR v.v.i. 2007-2010
// fir_mb_1x8()   fir filter batch with constatnt coefficients

void fir_mb_1x8(unsigned int nn, float * afmb, float * bfmb, float * zfd1,
		float * zfd2, float * zfd3, float * zfd4, float * zfd5, float * zfd6,
		float * zfd7, float * zfd8, float * zfz) {
	unsigned int i, j, k, n, ipn, nn2, nn3, nn4, nn5, nn6, nn7, nn8;
	float zs1[4], zs2[4], zs3[4], zs4[4], zs5[4], zs6[4], zs7[4], zs8[4];
	float *bfmbp1, *bfmbp2, *bfmbp3, *bfmbp4, *bfmbp5, *bfmbp6, *bfmbp7,
			*bfmbp8;

	for (n = 0; n < nn; n++) {
		nn2 = 2 * nn;
		nn3 = 3 * nn;
		nn4 = 4 * nn;
		nn5 = 5 * nn;
		nn6 = 6 * nn;
		nn7 = 7 * nn;
		nn8 = 8 * nn;

		for (i = 0; i < 4; i++) {
			zs1[i] = 0;
			zs2[i] = 0;
			zs3[i] = 0;
			zs4[i] = 0;
			zs5[i] = 0;
			zs6[i] = 0;
			zs7[i] = 0;
			zs8[i] = 0;
		}
		j = 0;
		bfmbp1 = &bfmb[0];
		bfmbp2 = &bfmb[nn];
		bfmbp3 = &bfmb[nn2];
		bfmbp4 = &bfmb[nn3];
		bfmbp5 = &bfmb[nn4];
		bfmbp6 = &bfmb[nn5];
		bfmbp7 = &bfmb[nn6];
		bfmbp8 = &bfmb[nn7];

		for (i = 0; i < nn; i++) {
			ipn = i + n;
			k = (ipn < nn) ? ipn : ipn - nn;
			zs1[j] += afmb[k] * *bfmbp1++;
			zs2[j] += afmb[k + nn] * *bfmbp2++;
			zs3[j] += afmb[k + nn2] * *bfmbp3++;
			zs4[j] += afmb[k + nn3] * *bfmbp4++;
			zs5[j] += afmb[k + nn4] * *bfmbp5++;
			zs6[j] += afmb[k + nn5] * *bfmbp6++;
			zs7[j] += afmb[k + nn6] * *bfmbp7++;
			zs8[j] += afmb[k + nn7] * *bfmbp8++;
			if (j < 3)
				j = j + 1;
			else
				j = 0;
		}
		zs1[0] += zs1[1];
		zs2[0] += zs2[1];
		zs3[0] += zs3[1];
		zs4[0] += zs4[1];
		zs5[0] += zs5[1];
		zs6[0] += zs6[1];
		zs7[0] += zs7[1];
		zs8[0] += zs8[1];
		zs1[2] += zs1[3];
		zs2[2] += zs2[3];
		zs3[2] += zs3[3];
		zs4[2] += zs4[3];
		zs5[2] += zs5[3];
		zs6[2] += zs6[3];
		zs7[2] += zs7[3];
		zs8[2] += zs8[3];
		zs1[0] += zs1[2];
		zs2[0] += zs2[2];
		zs3[0] += zs3[2];
		zs4[0] += zs4[2];
		zs5[0] += zs5[2];
		zs6[0] += zs6[2];
		zs7[0] += zs7[2];
		zs8[0] += zs8[2];

		zfz[n] = ((zs1[0] + zs2[0]) + (zs3[0] + zs4[0]))
				+ ((zs5[0] + zs6[0]) + (zs7[0] + zs8[0])); //product out

		bfmb[nn - 1 - n] = zfd1[n];
		bfmb[nn2 - 1 - n] = zfd2[n];
		bfmb[nn3 - 1 - n] = zfd3[n];
		bfmb[nn4 - 1 - n] = zfd4[n];
		bfmb[nn5 - 1 - n] = zfd5[n];
		bfmb[nn6 - 1 - n] = zfd6[n];
		bfmb[nn7 - 1 - n] = zfd7[n];
		bfmb[nn8 - 1 - n] = zfd8[n];
	}
}

//////////////////////////////////////////////////////////////////////////
// DSP_PLATFORM (c) UTIA AV CR v.v.i. 2007-2010
// bce_fp12_1x8_0_fir_mb_1x8_batch8()

void bce_fp12_1x8_0_fir_mb_1x8_batch8(unsigned int nn, unsigned int ii,
		float* afmb, float* bfmb, float* ufd, float* zfd) {

	unsigned int ii1, ii2, ii3, ii4, ii5, ii6, ii7, ii8, ii9, ii10, ii11, ii12,
			ii13, ii14, ii15;

	ii1 = (ii + 1) * nn;
	ii2 = (ii + 2) * nn;
	ii3 = (ii + 3) * nn;
	ii4 = (ii + 4) * nn;
	ii5 = (ii + 5) * nn;
	ii6 = (ii + 6) * nn;
	ii7 = (ii + 7) * nn;
	ii8 = (ii + 8) * nn;
	ii9 = (ii + 9) * nn;
	ii10 = (ii + 10) * nn;
	ii11 = (ii + 11) * nn;
	ii12 = (ii + 12) * nn;
	ii13 = (ii + 13) * nn;
	ii14 = (ii + 14) * nn;
	ii15 = (ii + 15) * nn;

	fir_mb_1x8(nn, &afmb[0], &bfmb[0], &ufd[ii8], &ufd[ii7], &ufd[ii6],
			&ufd[ii5], &ufd[ii4], &ufd[ii3], &ufd[ii2], &ufd[ii1], &zfd[0]);
	fir_mb_1x8(nn, &afmb[0], &bfmb[0], &ufd[ii9], &ufd[ii8], &ufd[ii7],
			&ufd[ii6], &ufd[ii5], &ufd[ii4], &ufd[ii3], &ufd[ii2], &zfd[nn]);
	fir_mb_1x8(nn, &afmb[0], &bfmb[0], &ufd[ii10], &ufd[ii9], &ufd[ii8],
			&ufd[ii7], &ufd[ii6], &ufd[ii5], &ufd[ii4], &ufd[ii3],
			&zfd[2 * nn]);
	fir_mb_1x8(nn, &afmb[0], &bfmb[0], &ufd[ii11], &ufd[ii10], &ufd[ii9],
			&ufd[ii8], &ufd[ii7], &ufd[ii6], &ufd[ii5], &ufd[ii4],
			&zfd[3 * nn]);
	fir_mb_1x8(nn, &afmb[0], &bfmb[0], &ufd[ii12], &ufd[ii11], &ufd[ii10],
			&ufd[ii9], &ufd[ii8], &ufd[ii7], &ufd[ii6], &ufd[ii5],
			&zfd[4 * nn]);
	fir_mb_1x8(nn, &afmb[0], &bfmb[0], &ufd[ii13], &ufd[ii12], &ufd[ii11],
			&ufd[ii10], &ufd[ii9], &ufd[ii8], &ufd[ii7], &ufd[ii6],
			&zfd[5 * nn]);
	fir_mb_1x8(nn, &afmb[0], &bfmb[0], &ufd[ii14], &ufd[ii13], &ufd[ii12],
			&ufd[ii11], &ufd[ii10], &ufd[ii9], &ufd[ii8], &ufd[ii7],
			&zfd[6 * nn]);
	fir_mb_1x8(nn, &afmb[0], &bfmb[0], &ufd[ii15], &ufd[ii14], &ufd[ii13],
			&ufd[ii12], &ufd[ii11], &ufd[ii10], &ufd[ii9], &ufd[ii8],
			&zfd[7 * nn]);
}

//////////////////////////////////////////////////////////////////////////
// DSP_PLATFORM (c) UTIA AV CR v.v.i. 2007-2010
// lms_mu_mb_1x8()   Adaptive LMS fir filter batch

void lms_mu_mb_1x8(unsigned int nn, float * af_mb, float * bfmb, float * zfd1,
		float * zfd2, float * zfd3, float * zfd4, float * zfd5, float * zfd6,
		float * zfd7, float * zfd8, float * zfz, float * zf_mb, float * mu) {
	unsigned int i, j, k, n, ipn, nn2, nn3, nn4, nn5, nn6, nn7, nn8;
	float zs1[4], zs2[4], zs3[4], zs4[4], zs5[4], zs6[4], zs7[4], zs8[4];
	float *bfmbp1, *bfmbp2, *bfmbp3, *bfmbp4, *bfmbp5, *bfmbp6, *bfmbp7,
			*bfmbp8;
	float e, mue;

	for (n = 0; n < nn; n++) {
		nn2 = 2 * nn;
		nn3 = 3 * nn;
		nn4 = 4 * nn;
		nn5 = 5 * nn;
		nn6 = 6 * nn;
		nn7 = 7 * nn;
		nn8 = 8 * nn;
		for (i = 0; i < 4; i++) {
			zs1[i] = 0;
			zs2[i] = 0;
			zs3[i] = 0;
			zs4[i] = 0;
			zs5[i] = 0;
			zs6[i] = 0;
			zs7[i] = 0;
			zs8[i] = 0;
		}

		j = 0;
		bfmbp1 = &bfmb[0];
		bfmbp2 = &bfmb[nn];
		bfmbp3 = &bfmb[nn2];
		bfmbp4 = &bfmb[nn3];
		bfmbp5 = &bfmb[nn4];
		bfmbp6 = &bfmb[nn5];
		bfmbp7 = &bfmb[nn6];
		bfmbp8 = &bfmb[nn7];
		for (i = 0; i < nn; i++) {
			ipn = i + n;
			k = (ipn < nn) ? ipn : ipn - nn;
			zs1[j] += af_mb[k] * *bfmbp1++;
			zs2[j] += af_mb[k + nn] * *bfmbp2++;
			zs3[j] += af_mb[k + nn2] * *bfmbp3++;
			zs4[j] += af_mb[k + nn3] * *bfmbp4++;
			zs5[j] += af_mb[k + nn4] * *bfmbp5++;
			zs6[j] += af_mb[k + nn5] * *bfmbp6++;
			zs7[j] += af_mb[k + nn6] * *bfmbp7++;
			zs8[j] += af_mb[k + nn7] * *bfmbp8++;
			if (j < 3)
				j = j + 1;
			else
				j = 0;
		}

		zs1[0] += zs1[1];
		zs2[0] += zs2[1];
		zs3[0] += zs3[1];
		zs4[0] += zs4[1];
		zs5[0] += zs5[1];
		zs6[0] += zs6[1];
		zs7[0] += zs7[1];
		zs8[0] += zs8[1];
		zs1[2] += zs1[3];
		zs2[2] += zs2[3];
		zs3[2] += zs3[3];
		zs4[2] += zs4[3];
		zs5[2] += zs5[3];
		zs6[2] += zs6[3];
		zs7[2] += zs7[3];
		zs8[2] += zs8[3];
		zs1[0] += zs1[2];
		zs2[0] += zs2[2];
		zs3[0] += zs3[2];
		zs4[0] += zs4[2];
		zs5[0] += zs5[2];
		zs6[0] += zs6[2];
		zs7[0] += zs7[2];
		zs8[0] += zs8[2];

		zf_mb[n] = ((zs1[0] + zs2[0]) + (zs3[0] + zs4[0]))
				+ ((zs5[0] + zs6[0]) + (zs7[0] + zs8[0])); //product out
		e = zfz[n] - zf_mb[n];
		mue = mu[0] * e;

		if (n == 0) {
			for (i = 0; i < nn; i++) {
				af_mb[i] += (mue * bfmb[i]);
				af_mb[i + nn] += (mue * bfmb[i + nn]);
				af_mb[i + nn2] += (mue * bfmb[i + nn2]);
				af_mb[i + nn3] += (mue * bfmb[i + nn3]);
				af_mb[i + nn4] += (mue * bfmb[i + nn4]);
				af_mb[i + nn5] += (mue * bfmb[i + nn5]);
				af_mb[i + nn6] += (mue * bfmb[i + nn6]);
				af_mb[i + nn7] += (mue * bfmb[i + nn7]);
			}
		} else {
			for (i = 0; i < (nn - n); i++) {
				af_mb[n + i] += (mue * bfmb[i]);
				af_mb[n + i + nn] += (mue * bfmb[i + nn]);
				af_mb[n + i + nn2] += (mue * bfmb[i + nn2]);
				af_mb[n + i + nn3] += (mue * bfmb[i + nn3]);
				af_mb[n + i + nn4] += (mue * bfmb[i + nn4]);
				af_mb[n + i + nn5] += (mue * bfmb[i + nn5]);
				af_mb[n + i + nn6] += (mue * bfmb[i + nn6]);
				af_mb[n + i + nn7] += (mue * bfmb[i + nn7]);
			}
			for (i = 0; i < n; i++) {
				af_mb[i] += (mue * bfmb[nn - n + i]);
				af_mb[i + nn] += (mue * bfmb[nn - n + i + nn]);
				af_mb[i + nn2] += (mue * bfmb[nn - n + i + nn2]);
				af_mb[i + nn3] += (mue * bfmb[nn - n + i + nn3]);
				af_mb[i + nn4] += (mue * bfmb[nn - n + i + nn4]);
				af_mb[i + nn5] += (mue * bfmb[nn - n + i + nn5]);
				af_mb[i + nn6] += (mue * bfmb[nn - n + i + nn6]);
				af_mb[i + nn7] += (mue * bfmb[nn - n + i + nn7]);
			}
		}
		bfmb[nn - 1 - n] = zfd1[n];
		bfmb[nn2 - 1 - n] = zfd2[n];
		bfmb[nn3 - 1 - n] = zfd3[n];
		bfmb[nn4 - 1 - n] = zfd4[n];
		bfmb[nn5 - 1 - n] = zfd5[n];
		bfmb[nn6 - 1 - n] = zfd6[n];
		bfmb[nn7 - 1 - n] = zfd7[n];
		bfmb[nn8 - 1 - n] = zfd8[n];
	}
}
//////////////////////////////////////////////////////////////////////////
// DSP_PLATFORM (c) UTIA AV CR v.v.i. 2007-2010
// bce_fp12_1x8_0_lms_mb_1x8_batch8()

void bce_fp12_1x8_0_lms_mb_1x8_batch8(unsigned nn, unsigned ii, float* ufd,
		float* zfd, float* mu, float* af_mu_mb, float* bf_mu_mb,
		float* zf_mu_mb2, float* zf_mu_mb3) {

	unsigned int ii1, ii2, ii3, ii4, ii5, ii6, ii7, ii8, ii9, ii10, ii11, ii12,
			ii13, ii14, ii15;

	ii1 = (ii + 1) * nn;
	ii2 = (ii + 2) * nn;
	ii3 = (ii + 3) * nn;
	ii4 = (ii + 4) * nn;
	ii5 = (ii + 5) * nn;
	ii6 = (ii + 6) * nn;
	ii7 = (ii + 7) * nn;
	ii8 = (ii + 8) * nn;
	ii9 = (ii + 9) * nn;
	ii10 = (ii + 10) * nn;
	ii11 = (ii + 11) * nn;
	ii12 = (ii + 12) * nn;
	ii13 = (ii + 13) * nn;
	ii14 = (ii + 14) * nn;
	ii15 = (ii + 15) * nn;

	lms_mu_mb_1x8(nn, &af_mu_mb[0], &bf_mu_mb[0], &ufd[ii8], &ufd[ii7],
			&ufd[ii6], &ufd[ii5], &ufd[ii4], &ufd[ii3], &ufd[ii2], &ufd[ii1],
			&zfd[0], &zf_mu_mb2[0], mu);
	lms_mu_mb_1x8(nn, &af_mu_mb[0], &bf_mu_mb[0], &ufd[ii9], &ufd[ii8],
			&ufd[ii7], &ufd[ii6], &ufd[ii5], &ufd[ii4], &ufd[ii3], &ufd[ii2],
			&zfd[nn], &zf_mu_mb3[0], mu);
	lms_mu_mb_1x8(nn, &af_mu_mb[0], &bf_mu_mb[0], &ufd[ii10], &ufd[ii9],
			&ufd[ii8], &ufd[ii7], &ufd[ii6], &ufd[ii5], &ufd[ii4], &ufd[ii3],
			&zfd[2 * nn], &zf_mu_mb2[0], mu);
	lms_mu_mb_1x8(nn, &af_mu_mb[0], &bf_mu_mb[0], &ufd[ii11], &ufd[ii10],
			&ufd[ii9], &ufd[ii8], &ufd[ii7], &ufd[ii6], &ufd[ii5], &ufd[ii4],
			&zfd[3 * nn], &zf_mu_mb3[0], mu);
	lms_mu_mb_1x8(nn, &af_mu_mb[0], &bf_mu_mb[0], &ufd[ii12], &ufd[ii11],
			&ufd[ii10], &ufd[ii9], &ufd[ii8], &ufd[ii7], &ufd[ii6], &ufd[ii5],
			&zfd[4 * nn], &zf_mu_mb2[0], mu);
	lms_mu_mb_1x8(nn, &af_mu_mb[0], &bf_mu_mb[0], &ufd[ii13], &ufd[ii12],
			&ufd[ii11], &ufd[ii10], &ufd[ii9], &ufd[ii8], &ufd[ii7], &ufd[ii6],
			&zfd[5 * nn], &zf_mu_mb3[0], mu);
	lms_mu_mb_1x8(nn, &af_mu_mb[0], &bf_mu_mb[0], &ufd[ii14], &ufd[ii13],
			&ufd[ii12], &ufd[ii11], &ufd[ii10], &ufd[ii9], &ufd[ii8], &ufd[ii7],
			&zfd[6 * nn], &zf_mu_mb2[0], mu);
	lms_mu_mb_1x8(nn, &af_mu_mb[0], &bf_mu_mb[0], &ufd[ii15], &ufd[ii14],
			&ufd[ii13], &ufd[ii12], &ufd[ii11], &ufd[ii10], &ufd[ii9],
			&ufd[ii8], &zfd[7 * nn], &zf_mu_mb3[0], mu);
}

void bce_fp12_1x8_0_fir_1x8_write(unsigned int n, unsigned int dh, float* a,
		float*b, float* afd1, float* afd2, float* afd3, float* afd4,
		float* afd5, float* afd6, float* afd7, float* afd8) {
	unsigned int n2, n3, n4, n5, n6, n7;

	n2 = 2 * n;
	n3 = 3 * n;
	n4 = 4 * n;
	n5 = 5 * n;
	n6 = 6 * n;
	n7 = 7 * n;

	// Write A (with wrap)
	wal_mb2dmem(worker1, 0, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[0], n);
	wal_mb2dmem(worker1, 1, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n], n);
	wal_mb2dmem(worker1, 2, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n2], n);
	wal_mb2dmem(worker1, 3, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n3], n);
	wal_mb2dmem(worker1, 4, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n4], n);
	wal_mb2dmem(worker1, 5, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n5], n);
	wal_mb2dmem(worker1, 6, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n6], n);
	wal_mb2dmem(worker1, 7, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n7], n);

	// Write input data
	wal_mb2dmem(worker1, 0, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd1[0], n);
	wal_mb2dmem(worker1, 1, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd2[0], n);
	wal_mb2dmem(worker1, 2, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd3[0], n);
	wal_mb2dmem(worker1, 3, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd4[0], n);
	wal_mb2dmem(worker1, 4, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd5[0], n);
	wal_mb2dmem(worker1, 5, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd6[0], n);
	wal_mb2dmem(worker1, 6, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd7[0], n);
	wal_mb2dmem(worker1, 7, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd8[0], n);

	// Write B
	wal_mb2dmem(worker1, 0, WAL_BCE_JK_DMEM_B, 0x000, &b[0], n);
	wal_mb2dmem(worker1, 1, WAL_BCE_JK_DMEM_B, 0x000, &b[n], n);
	wal_mb2dmem(worker1, 2, WAL_BCE_JK_DMEM_B, 0x000, &b[n2], n);
	wal_mb2dmem(worker1, 3, WAL_BCE_JK_DMEM_B, 0x000, &b[n3], n);
	wal_mb2dmem(worker1, 4, WAL_BCE_JK_DMEM_B, 0x000, &b[n4], n);
	wal_mb2dmem(worker1, 5, WAL_BCE_JK_DMEM_B, 0x000, &b[n5], n);
	wal_mb2dmem(worker1, 6, WAL_BCE_JK_DMEM_B, 0x000, &b[n6], n);
	wal_mb2dmem(worker1, 7, WAL_BCE_JK_DMEM_B, 0x000, &b[n7], n);
}

void bce_fp12_1x8_0_fir_1x8_write_loop(unsigned int n, unsigned int dh,
		float* afd1, float* afd2, float* afd3, float* afd4, float* afd5,
		float* afd6, float* afd7, float* afd8) {

	// Write input data
	wal_mb2dmem(worker1, 0, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd1[0], n);
	wal_mb2dmem(worker1, 1, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd2[0], n);
	wal_mb2dmem(worker1, 2, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd3[0], n);
	wal_mb2dmem(worker1, 3, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd4[0], n);
	wal_mb2dmem(worker1, 4, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd5[0], n);
	wal_mb2dmem(worker1, 5, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd6[0], n);
	wal_mb2dmem(worker1, 6, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd7[0], n);
	wal_mb2dmem(worker1, 7, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd8[0], n);
}

void bce_fp12_1x8_0_fir_1x8_read(unsigned int n, unsigned int dh, float* w,
		float* u, float* z) {

	unsigned int n2, n3, n4, n5, n6, n7;
	n2 = 2 * n;
	n3 = 3 * n;
	n4 = 4 * n;
	n5 = 5 * n;
	n6 = 6 * n;
	n7 = 7 * n;

	wal_dmem2mb(worker1, 0, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[0], n);
	wal_dmem2mb(worker1, 1, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n], n);
	wal_dmem2mb(worker1, 2, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n2], n);
	wal_dmem2mb(worker1, 3, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n3], n);
	wal_dmem2mb(worker1, 4, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n4], n);
	wal_dmem2mb(worker1, 5, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n5], n);
	wal_dmem2mb(worker1, 6, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n6], n);
	wal_dmem2mb(worker1, 7, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n7], n);

	wal_dmem2mb(worker1, 0, WAL_BCE_JK_DMEM_B, 0x000, &u[0], n);
	wal_dmem2mb(worker1, 1, WAL_BCE_JK_DMEM_B, 0x000, &u[n], n);
	wal_dmem2mb(worker1, 2, WAL_BCE_JK_DMEM_B, 0x000, &u[n2], n);
	wal_dmem2mb(worker1, 3, WAL_BCE_JK_DMEM_B, 0x000, &u[n3], n);
	wal_dmem2mb(worker1, 4, WAL_BCE_JK_DMEM_B, 0x000, &u[n4], n);
	wal_dmem2mb(worker1, 5, WAL_BCE_JK_DMEM_B, 0x000, &u[n5], n);
	wal_dmem2mb(worker1, 6, WAL_BCE_JK_DMEM_B, 0x000, &u[n6], n);
	wal_dmem2mb(worker1, 7, WAL_BCE_JK_DMEM_B, 0x000, &u[n7], n);

	wal_dmem2mb(worker1, 0, WAL_BCE_JK_DMEM_Z, dh * 0x100, &z[0], n);
}

void bce_fp12_1x8_0_fir_1x8_read_loop(unsigned int n, unsigned int dh, float* z) {

	wal_dmem2mb(worker1, 0, WAL_BCE_JK_DMEM_Z, dh * 0x100, &z[0], n);
}

//////////////////////////////////////////////////////////////////////////
// DSP_PLATFORM (c) UTIA AV CR v.v.i. 2007-2010
// bce_fp12_1x8_0_fir_pb_1x8_batch8()
//////////////////////////////////////////////////////////////////////////
//
//
// FIR filter with n coefficients, n step batch, n = max 255
// Supports fp01x1 fp01x8, fp01x4, fp01x8 accelerators
// Supports dp01x1 dp01x8, dp01x4         accelerators
//
// Input:
// A[]    = [w[0],w[1],..,w[n-1]] end of bank 0  FIR weights				(to get circular buffer from VPROD)
// A[]    = [u[0],u[1],..,u[n-1]] banks 2 or 3   System input				(two sets of system input data for ping pong)
// B[]    = [x[0],x[1],..,x[n-1]] banks 0        FIR filter data vector	(state from previous batch)
// B[]    = [u[0],u[1],..,u[n-1]] banks 2 or 3   System output				(two sets of system output data for ping pong)
//
// FIR filter section with wrap:
// y[0]   = [w[0],w[1],..,w[n-1]]        * [x[0],x[1],x[2],    ..,x[n-3],x[n-2],x[n-1]]
// y[1]   = [w[1],..,w[n-1],w[0]]        * [x[0],x[1],x[2],    ..,x[n-3],x[n-2],u[0]  ]
// y[2]   = [w[2],..,w[n-1],w[0],w[1]]   * [x[0],x[1],x[2],    ..,x[n-3],u[1],  u[0]  ]
// ..
// y[n-1] = [w[n-1],w[0],w[1],..,w[n-2]] * [x[0],u[n-2],u[n-3],..,u[2],  u[1],  u[0]  ]
// finaly form for the next batch of input data:
// [x[0],x[1],x[2],..,x[n-3],x[n-2],x[n-1]]=[u[n-1],u[n-2],u[n-3],..,u[2],u[1], u[0]  ]
//
// Output:
// B[]    = [x[0],x[1],..,x[n-1]] bank 0        (FIR state for the next batch)
// Z[]    = [y[0],y[1],..,y[n-1]] bank 2 or 3   (two sets of output filtered data for ping pong)
//
//
// DATA
//             [00],[01],.................,[fc],[fd],[fe],[ff]
// A bank 0			      <--------------w[]-------------->				    Filter weights
// A bank 1
// A bank 2	<------------u[i](2)-------------->		 						Block of input data
// A bank 3	<------------u[i](3)-------------->	     						Block of input data
//
// B bank 0 <------x[0:i-1],x[i:n-1]--------->								Data regressor
// B bank 1
// B bank 2
// B bank 3
//
// Z bank 0
// Z bank 1
// Z bank 2 <------------y[i](2)------------->                              Block of output data
// Z bank 3 <------------y[i](3)------------->								Block of output data
//
// COMPUTE
//
// y[i]		= w[]x[]	(y[i]     to Z     bank 2,3)	<-	VPROD		(w[]	A     bank 0  )	(x[]      B[]     bank 0  )
// x[n-i]	= u[i]		(u[n-i]	  to B     bank 0  )	<-  VA2B		(u[i]   A[i]  bank 2,3)
//
// VPROD can be replaced by VPROD_S2, VPROD_S4 or VPROD_S8 in case of SIMD accelerators.
// The weights w[] and state x[] of the FIR filter is split into 2, 4 or 8 parts with identical length.
// The SIMD version of VPROD (VPROD_S2, VPROD_S4 or VPROD_S8) is computing a single dot product y[i] = w'[]x[]
// and the resulting scalar y[i] is stored in Z bank 2 or Z bank 3 of all 2, 4 or 8 SIMD memories
// Z1,Z2 or Z1,Z2,Z3,Z4 or Z1,Z2,Z3,Z4,Z5,Z6,Z7,Z8
// Identical values of mu_e_a are computed in all parallel pages and therefore the LMS update of the splitted sections of w[]
// can be performed in parallel as well.
//
// The update of splitted state x[] is performed in parallel. Microblaze must support the correct placement of the system input blocks.
// This can be done in parallel with the LMS computation by the ping-pong method with dual I/O buffers in banks 2 and 3.
//
// Use of registers
// s0   counter of the main loop of the batch:     n,      n-1, .., 1
// s1   address from where to get data from Z:     0,        1, .., n-1
// s2   address in A, start VPROD with wrap:   256-n,  256-n+1, .., 255
// s3   length of the batch:                     n-1
// KH   value of s1 (before increment)
// KL   pointer to end of buffer (FF)
// SH   data bank (2) or (3) SH = _dh
// TMP  temporary value
// TMP2 prepare the next vector operation
//
// Use of memory
// _op operation VPROD or VPROD_S2 or VPROD_S4 or VPROD_S8
// _dh use bank 2 or bank 3
// _n
// _a  start of w[] a = 256 - n;
//
// Author:		Jiri Kadlec  (JK)
// Date:		01.06.2008
// Revisions:  03.08.2008   (JK)	Improved comments
//
//
//////////////////////////////////////////////////////////////////////////

void bce_fp12_1x8_0_fir_pb_1x8_batch8(int fw, int pbid, unsigned nn,
		unsigned ii, float* afpb, float* bfpb, float* ufd, float* zfd) {
	unsigned int ii1, ii2, ii3, ii4, ii5, ii6, ii7, ii8, ii9, ii10, ii11, ii12,
			ii13, ii14, ii15;

	ii1 = (ii + 1) * nn;
	ii2 = (ii + 2) * nn;
	ii3 = (ii + 3) * nn;
	ii4 = (ii + 4) * nn;
	ii5 = (ii + 5) * nn;
	ii6 = (ii + 6) * nn;
	ii7 = (ii + 7) * nn;
	ii8 = (ii + 8) * nn;
	ii9 = (ii + 9) * nn;
	ii10 = (ii + 10) * nn;
	ii11 = (ii + 11) * nn;
	ii12 = (ii + 12) * nn;
	ii13 = (ii + 13) * nn;
	ii14 = (ii + 14) * nn;
	ii15 = (ii + 15) * nn;

	bce_fp12_1x8_0_fir_1x8_write(nn, 2, &afpb[0], &bfpb[0], &ufd[ii8],
			&ufd[ii7], &ufd[ii6], &ufd[ii5], &ufd[ii4], &ufd[ii3], &ufd[ii2],
			&ufd[ii1]);
	wal_bce_jk_create_op(worker1, pbid, WAL_BCE_JK_VPROD_S8, nn);
	// unroll
	bce_fp12_1x8_0_fir_1x8_write_loop(nn, 3, &ufd[ii9], &ufd[ii8], &ufd[ii7],
			&ufd[ii6], &ufd[ii5], &ufd[ii4], &ufd[ii3], &ufd[ii2]);
	//sync1
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_fir_1x8_read_loop(nn, 2, &zfd[0]);
	bce_fp12_1x8_0_fir_1x8_write_loop(nn, 2, &ufd[ii10], &ufd[ii9], &ufd[ii8],
			&ufd[ii7], &ufd[ii6], &ufd[ii5], &ufd[ii4], &ufd[ii3]);
	//sync2
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_fir_1x8_read_loop(nn, 3, &zfd[nn]);
	bce_fp12_1x8_0_fir_1x8_write_loop(nn, 3, &ufd[ii11], &ufd[ii10], &ufd[ii9],
			&ufd[ii8], &ufd[ii7], &ufd[ii6], &ufd[ii5], &ufd[ii4]);
	//sync3
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_fir_1x8_read_loop(nn, 2, &zfd[2 * nn]);
	bce_fp12_1x8_0_fir_1x8_write_loop(nn, 2, &ufd[ii12], &ufd[ii11], &ufd[ii10],
			&ufd[ii9], &ufd[ii8], &ufd[ii7], &ufd[ii6], &ufd[ii5]);
	//sync4
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_fir_1x8_read_loop(nn, 3, &zfd[3 * nn]);
	bce_fp12_1x8_0_fir_1x8_write_loop(nn, 3, &ufd[ii13], &ufd[ii12], &ufd[ii11],
			&ufd[ii10], &ufd[ii9], &ufd[ii8], &ufd[ii7], &ufd[ii6]);
	//sync5
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_fir_1x8_read_loop(nn, 2, &zfd[4 * nn]);
	bce_fp12_1x8_0_fir_1x8_write_loop(nn, 2, &ufd[ii14], &ufd[ii13], &ufd[ii12],
			&ufd[ii11], &ufd[ii10], &ufd[ii9], &ufd[ii8], &ufd[ii7]);
	//sync6
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_fir_1x8_read_loop(nn, 3, &zfd[5 * nn]);
	bce_fp12_1x8_0_fir_1x8_write_loop(nn, 3, &ufd[ii15], &ufd[ii14], &ufd[ii13],
			&ufd[ii12], &ufd[ii11], &ufd[ii10], &ufd[ii9], &ufd[ii8]);
	//sync7
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_fir_1x8_read_loop(nn, 2, &zfd[6 * nn]);

	//sync8
	wal_pb2mb(worker1, NULL );

	// windup

	draw_pb2mb(worker1);

	wal_bce_jk_sync_operation(worker1);
	bce_fp12_1x8_0_fir_1x8_read(nn, 3, &afpb[0], &bfpb[0], &zfd[7 * nn]);
}

//////////////////////////////////////////////////////////////////////////
// DSP_PLATFORM (c) UTIA AV CR v.v.i. 2007-2010
// bce_fp12_1x8_0_lms_1x8_write()

void bce_fp12_1x8_0_lms_1x8_write(unsigned int n, unsigned int dh, float* a,
		float*b, float* afd1, float* afd2, float* afd3, float* afd4,
		float* afd5, float* afd6, float* afd7, float* afd8, float* zfmb,
		float* mu) {
	unsigned int n2, n3, n4, n5, n6, n7;

	n2 = 2 * n;
	n3 = 3 * n;
	n4 = 4 * n;
	n5 = 5 * n;
	n6 = 6 * n;
	n7 = 7 * n;

	// Write A (with wrap)
	wal_mb2dmem(worker1, 0, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[0], n);
	wal_mb2dmem(worker1, 1, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n], n);
	wal_mb2dmem(worker1, 2, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n2], n);
	wal_mb2dmem(worker1, 3, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n3], n);
	wal_mb2dmem(worker1, 4, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n4], n);
	wal_mb2dmem(worker1, 5, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n5], n);
	wal_mb2dmem(worker1, 6, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n6], n);
	wal_mb2dmem(worker1, 7, WAL_BCE_JK_DMEM_A, 0x100 - n, &a[n7], n);

	// Write input data
	wal_mb2dmem(worker1, 0, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd1[0], n);
	wal_mb2dmem(worker1, 1, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd2[0], n);
	wal_mb2dmem(worker1, 2, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd3[0], n);
	wal_mb2dmem(worker1, 3, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd4[0], n);
	wal_mb2dmem(worker1, 4, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd5[0], n);
	wal_mb2dmem(worker1, 5, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd6[0], n);
	wal_mb2dmem(worker1, 6, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd7[0], n);
	wal_mb2dmem(worker1, 7, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd8[0], n);

	// Write B
	wal_mb2dmem(worker1, 0, WAL_BCE_JK_DMEM_B, 0x000, &b[0], n);
	wal_mb2dmem(worker1, 1, WAL_BCE_JK_DMEM_B, 0x000, &b[n], n);
	wal_mb2dmem(worker1, 2, WAL_BCE_JK_DMEM_B, 0x000, &b[n2], n);
	wal_mb2dmem(worker1, 3, WAL_BCE_JK_DMEM_B, 0x000, &b[n3], n);
	wal_mb2dmem(worker1, 4, WAL_BCE_JK_DMEM_B, 0x000, &b[n4], n);
	wal_mb2dmem(worker1, 5, WAL_BCE_JK_DMEM_B, 0x000, &b[n5], n);
	wal_mb2dmem(worker1, 6, WAL_BCE_JK_DMEM_B, 0x000, &b[n6], n);
	wal_mb2dmem(worker1, 7, WAL_BCE_JK_DMEM_B, 0x000, &b[n7], n);

	// Write Z_MB
	wal_mb2dmem(worker1, 0, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 1, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 2, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 3, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 4, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 5, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 6, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 7, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);

	// Write mu
	wal_mb2dmem(worker1, 0, WAL_BCE_JK_DMEM_Z, 0x0ff, &mu[0], 1);
	wal_mb2dmem(worker1, 1, WAL_BCE_JK_DMEM_Z, 0x0ff, &mu[0], 1);
	wal_mb2dmem(worker1, 2, WAL_BCE_JK_DMEM_Z, 0x0ff, &mu[0], 1);
	wal_mb2dmem(worker1, 3, WAL_BCE_JK_DMEM_Z, 0x0ff, &mu[0], 1);
	wal_mb2dmem(worker1, 4, WAL_BCE_JK_DMEM_Z, 0x0ff, &mu[0], 1);
	wal_mb2dmem(worker1, 5, WAL_BCE_JK_DMEM_Z, 0x0ff, &mu[0], 1);
	wal_mb2dmem(worker1, 6, WAL_BCE_JK_DMEM_Z, 0x0ff, &mu[0], 1);
	wal_mb2dmem(worker1, 7, WAL_BCE_JK_DMEM_Z, 0x0ff, &mu[0], 1);
}

//////////////////////////////////////////////////////////////////////////
// DSP_PLATFORM (c) UTIA AV CR v.v.i. 2007-2010
// bce_fp12_1x8_0_lms_1x8_write_loop()

void bce_fp12_1x8_0_lms_1x8_write_loop(unsigned int n, unsigned int dh,
		float* afd1, float* afd2, float* afd3, float* afd4, float* afd5,
		float* afd6, float* afd7, float* afd8, float* zfmb) {

	// Write input data
	wal_mb2dmem(worker1, 0, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd1[0], n);
	wal_mb2dmem(worker1, 1, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd2[0], n);
	wal_mb2dmem(worker1, 2, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd3[0], n);
	wal_mb2dmem(worker1, 3, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd4[0], n);
	wal_mb2dmem(worker1, 4, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd5[0], n);
	wal_mb2dmem(worker1, 5, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd6[0], n);
	wal_mb2dmem(worker1, 6, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd7[0], n);
	wal_mb2dmem(worker1, 7, WAL_BCE_JK_DMEM_A, dh * 0x100, &afd8[0], n);

	// Write Z_MB
	wal_mb2dmem(worker1, 0, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 1, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 2, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 3, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 4, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 5, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 6, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
	wal_mb2dmem(worker1, 7, WAL_BCE_JK_DMEM_B, dh * 0x100, &zfmb[0], n);
}

//////////////////////////////////////////////////////////////////////////
// DSP_PLATFORM (c) UTIA AV CR v.v.i. 2007-2010
// bce_fp12_1x8_0_lms_1x8_read()

void bce_fp12_1x8_0_lms_1x8_read(unsigned int n, unsigned int dh, float* w,
		float* u, float* z) {

	unsigned int n2, n3, n4, n5, n6, n7;
	n2 = 2 * n;
	n3 = 3 * n;
	n4 = 4 * n;
	n5 = 5 * n;
	n6 = 6 * n;
	n7 = 7 * n;

	wal_dmem2mb(worker1, 0, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[0], n);
	wal_dmem2mb(worker1, 1, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n], n);
	wal_dmem2mb(worker1, 2, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n2], n);
	wal_dmem2mb(worker1, 3, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n3], n);
	wal_dmem2mb(worker1, 4, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n4], n);
	wal_dmem2mb(worker1, 5, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n5], n);
	wal_dmem2mb(worker1, 6, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n6], n);
	wal_dmem2mb(worker1, 7, WAL_BCE_JK_DMEM_A, 0x100 - n, &w[n7], n);

	wal_dmem2mb(worker1, 0, WAL_BCE_JK_DMEM_B, 0x000, &u[0], n);
	wal_dmem2mb(worker1, 1, WAL_BCE_JK_DMEM_B, 0x000, &u[n], n);
	wal_dmem2mb(worker1, 2, WAL_BCE_JK_DMEM_B, 0x000, &u[n2], n);
	wal_dmem2mb(worker1, 3, WAL_BCE_JK_DMEM_B, 0x000, &u[n3], n);
	wal_dmem2mb(worker1, 4, WAL_BCE_JK_DMEM_B, 0x000, &u[n4], n);
	wal_dmem2mb(worker1, 5, WAL_BCE_JK_DMEM_B, 0x000, &u[n5], n);
	wal_dmem2mb(worker1, 6, WAL_BCE_JK_DMEM_B, 0x000, &u[n6], n);
	wal_dmem2mb(worker1, 7, WAL_BCE_JK_DMEM_B, 0x000, &u[n7], n);

	wal_dmem2mb(worker1, 0, WAL_BCE_JK_DMEM_Z, dh * 0x100, &z[0], n);
}

void bce_fp12_1x8_0_lms_1x8_read_loop(unsigned int n, unsigned int dh, float* z) {

	wal_dmem2mb(worker1, 0, WAL_BCE_JK_DMEM_Z, dh * 0x100, &z[0], n);
}

//////////////////////////////////////////////////////////////////////////
// DSP_PLATFORM (c) UTIA AV CR v.v.i. 2007-2010
// bce_fp12_1x8_0_lms_pb_1x8_batch8()
//////////////////////////////////////////////////////////////////////////
//
// LMS filter with n coefficients, n step batch, n = max 255
// Supports fp01x1 fp01x2, fp01x4, fp01x8 accelerators
// Supports dp01x1 dp01x2, dp01x4         accelerators
//
// Input:
// A[]    = [w[0],w[1],..,w[n-1]] end of bank 0  LMS weights				(to get circular buffer from VPROD)
// A[]    = [u[0],u[1],..,u[n-1]] banks 2 or 3   System input				(two sets of system input data for ping pong)
// B[]    = [x[0],x[1],..,x[n-1]] banks 0        FIR filter data vector		(state from previous batch)
// B[]    = [u[0],u[1],..,u[n-1]] banks 2 or 3   System output				(two sets of system output data for ping pong)
//
// FIR filter section with wrap:
// y[0]   = [w[0],w[1],..,w[n-1]]        * [x[0],x[1],x[2],    ..,x[n-3],x[n-2],x[n-1]]
// y[1]   = [w[1],..,w[n-1],w[0]]        * [x[0],x[1],x[2],    ..,x[n-3],x[n-2],u[0]  ]
// y[2]   = [w[2],..,w[n-1],w[0],w[1]]   * [x[0],x[1],x[2],    ..,x[n-3],u[1],  u[0]  ]
// ..
// y[n-1] = [w[n-1],w[0],w[1],..,w[n-2]] * [x[0],u[n-2],u[n-3],..,u[2],  u[1],  u[0]  ]
// finaly form for the next batch of input data:
// [x[0],x[1],x[2],..,x[n-3],x[n-2],x[n-1]]=[u[n-1],u[n-2],u[n-3],..,u[2],u[1], u[0]  ]
//
// Output:
// A[]    = [w[0],w[1],..,w[n-1]] end of bank 0 (to get circular buffer from VPROD)
// B[]    = [x[0],x[1],..,x[n-1]] bank 0        (FIR state for the next batch)
// Z[]    = [y[0],y[1],..,y[n-1]] bank 2 or 3   (two sets of output filtered data for ping pong)
//
//
// DATA
//             [00],[01],.................,[fc],[fd],[fe],[ff]
// A bank 0			      <--------------w[]-------------->				    Filter weights
// A bank 1                                             A[1ff]=mu_e_a
// A bank 2	<------------u[i](2)-------------->		    A[2ff]=e(2)			Block of input data
// A bank 3	<------------u[i](3)-------------->	        A[3ff]=e(3)			Block of input data
//
// B bank 0 <------x[0:i-1],x[i:n-1]--------->								Data regressor
// B bank 1			      <--------------wb[]------------->
// B bank 2 <------------d[i](2)------------->          B[2ff]=mu_e_b(2)
// B bank 3 <------------d[i](3)------------->          B[3ff]=mu_e_b(3)
//
// Z bank 0                                             Z[0ff]=mu
// Z bank 1               <-w_lms[0:n-i-1],w_lms[n-i:n-1]->
// Z bank 2 <------------y[i](2)------------->                              Block of output data
// Z bank 3 <------------y[i](3)------------->								Block of output data
//
// COMPUTE
//
// y[i]			  = w[]x[]				(y[i]     to Z     bank 2,3)	<-	VPROD		(w[]	A     bank 0  )	(x[]      B[]     bank 0  )
// e			  = d[i]-y[i]			(e		  to A[ff] bank 2,3)	<-  VSUB_BZ2A	(d[i]	B[i]  bank 2,3)	(y[i]     Z[i]    bank 2,3)
// mu_e_b		  = e * mu				(mu_e_b   to B[ff] bank 2,3)	<-	VMULT_AZ2B	(e		A[ff] bank 2,3)	(mu       Z[ff]   bank 0  )
// mu_e_a		  = mu_e_b				(mu_e_a   to A[ff] bank 1  )	<-	VB2A		(mu_e_b B[ff] bank 2,3)
// w_lms[n-i:n-1] = mu_e_a * x[0:i-1]	(w_lms[]  to Z     bank 1  )	<-  VMULT		(mu_e_a A[ff] bank 1  )	(x[0:i-1] B[]	  bank 0  )
// w_lms[0:n-i-1] = mu_e_a * x[i:n-1]	(w_lms[]  to Z     bank 1  )	<-	VMULT		(mu_e_a A[ff] bank 1  )	(x[i:n-1] B[]     bank 0  )
// w_b[]		  = w[]					(w_b[]	  to B     bank 1  )	<-	VB2A		(w[]	A     bank 0  )
// w[]			  = w_b[] + w_lms[]		(w[]	  to A     bank 0  )	<-	VADD_BZ2A	(w_b[]  B     bank 1  )	(w_lms[]  Z[]	  bank 1  )
// x[n-i]		  = u[i]				(u[n-i]	  to B     bank 0  )	<-  VA2B		(u[i]   A[i]  bank 2,3)
//
//
// VPROD can be replaced by VPROD_S2, VPROD_S4 or VPROD_S8 in case of SIMD accelerators.
// The weights w[] and state x[] of the FIR filter is split into 2, 4 or 8 parts with identical length.
// The SIMD version of VPROD (VPROD_S2, VPROD_S4 or VPROD_S8) is computing a single dot product y[i] = w'[]x[]
// and the resulting scalar y[i] is stored in Z bank 2 or Z bank 3 of all 2, 4 or 8 SIMD memories
// Z1,Z2 or Z1,Z2,Z3,Z4 or Z1,Z2,Z3,Z4,Z5,Z6,Z7,Z8
// Identical values of mu_e_a are computed in all parallel pages and therefore the LMS update of the splitted sections of w[]
// can be performed in parallel as well.
//
// The update of splitted state x[] is performed in parallel. Microblaze must support the correct placement of the system input blocks.
// This can be done in parallel with the LMS computation by the ping-pong method with dual I/O buffers in banks 2 and 3.
//
// Use of registers
// s0   counter of the main loop of the batch:     n,      n-1, .., 1
// s1   address from where to get data from Z:     0,        1, .., n-1
// s2   address in A, start VPROD with wrap:   256-n,  256-n+1, .., 255
// s3   length of the batch:                     n-1
// KH   value of s1 (before increment)
// KL   pointer to end of buffer (FF)
// SH   data bank (2) or (3) SH = _dh
// TMP  temporary value
// TMP2 prepare the next vector operation
//
// Use of memory
// _op operation VPROD or VPROD_S2 or VPROD_S4 or VPROD_S8
// _dh use bank 2 or bank 3
// _n
// _a  start of w[] a = 256 - n;
//
//////////////////////////////////////////////////////////////////////////

void bce_fp12_1x8_0_lms_pb_1x8_batch8(int fw, int pbid, unsigned nn,
		unsigned ii, float* ufd, float* zfd, float* mu, float* af_mu_pb,
		float* bf_mu_pb, float* zf_mu_pb2, float* zf_mu_pb3) {
	unsigned int ii1, ii2, ii3, ii4, ii5, ii6, ii7, ii8, ii9, ii10, ii11, ii12,
			ii13, ii14, ii15;

	ii1 = (ii + 1) * nn;
	ii2 = (ii + 2) * nn;
	ii3 = (ii + 3) * nn;
	ii4 = (ii + 4) * nn;
	ii5 = (ii + 5) * nn;
	ii6 = (ii + 6) * nn;
	ii7 = (ii + 7) * nn;
	ii8 = (ii + 8) * nn;
	ii9 = (ii + 9) * nn;
	ii10 = (ii + 10) * nn;
	ii11 = (ii + 11) * nn;
	ii12 = (ii + 12) * nn;
	ii13 = (ii + 13) * nn;
	ii14 = (ii + 14) * nn;
	ii15 = (ii + 15) * nn;

	bce_fp12_1x8_0_lms_1x8_write(nn, 2, &af_mu_pb[0], &bf_mu_pb[0], &ufd[ii8],
			&ufd[ii7], &ufd[ii6], &ufd[ii5], &ufd[ii4], &ufd[ii3], &ufd[ii2],
			&ufd[ii1], &zfd[0], mu);
	wal_bce_jk_create_op(worker1, pbid, WAL_BCE_JK_VPROD_S8, nn);
	// unroll
	bce_fp12_1x8_0_lms_1x8_write_loop(nn, 3, &ufd[ii9], &ufd[ii8], &ufd[ii7],
			&ufd[ii6], &ufd[ii5], &ufd[ii4], &ufd[ii3], &ufd[ii2], &zfd[nn]);
	//sync1
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_lms_1x8_read_loop(nn, 2, &zf_mu_pb2[0]);
	bce_fp12_1x8_0_lms_1x8_write_loop(nn, 2, &ufd[ii10], &ufd[ii9], &ufd[ii8],
			&ufd[ii7], &ufd[ii6], &ufd[ii5], &ufd[ii4], &ufd[ii3],
			&zfd[2 * nn]);
	//sync2
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_lms_1x8_read_loop(nn, 3, &zf_mu_pb3[0]);
	bce_fp12_1x8_0_lms_1x8_write_loop(nn, 3, &ufd[ii11], &ufd[ii10], &ufd[ii9],
			&ufd[ii8], &ufd[ii7], &ufd[ii6], &ufd[ii5], &ufd[ii4],
			&zfd[3 * nn]);

	//sync3
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_lms_1x8_read_loop(nn, 2, &zf_mu_pb2[0]);
	bce_fp12_1x8_0_lms_1x8_write_loop(nn, 2, &ufd[ii12], &ufd[ii11], &ufd[ii10],
			&ufd[ii9], &ufd[ii8], &ufd[ii7], &ufd[ii6], &ufd[ii5],
			&zfd[4 * nn]);

	//sync4
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_lms_1x8_read_loop(nn, 3, &zf_mu_pb3[0]);
	bce_fp12_1x8_0_lms_1x8_write_loop(nn, 3, &ufd[ii13], &ufd[ii12], &ufd[ii11],
			&ufd[ii10], &ufd[ii9], &ufd[ii8], &ufd[ii7], &ufd[ii6],
			&zfd[5 * nn]);

	//sync5
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_lms_1x8_read_loop(nn, 2, &zf_mu_pb2[0]);
	bce_fp12_1x8_0_lms_1x8_write_loop(nn, 2, &ufd[ii14], &ufd[ii13], &ufd[ii12],
			&ufd[ii11], &ufd[ii10], &ufd[ii9], &ufd[ii8], &ufd[ii7],
			&zfd[6 * nn]);

	//sync6
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_lms_1x8_read_loop(nn, 3, &zf_mu_pb3[0]);
	bce_fp12_1x8_0_lms_1x8_write_loop(nn, 3, &ufd[ii15], &ufd[ii14], &ufd[ii13],
			&ufd[ii12], &ufd[ii11], &ufd[ii10], &ufd[ii9], &ufd[ii8],
			&zfd[7 * nn]);

	//sync7
	wal_pb2mb(worker1, NULL );
	bce_fp12_1x8_0_lms_1x8_read_loop(nn, 2, &zf_mu_pb2[0]);

	//sync8
	wal_pb2mb(worker1, NULL );

	// windup

	draw_pb2mb(worker1);

	wal_bce_jk_sync_operation(worker1);
	bce_fp12_1x8_0_lms_1x8_read(nn, 3, &af_mu_pb[0], &bf_mu_pb[0],
			&zf_mu_pb3[0]);
}

int edkdsp_fir_lms() {
	int fw = -1;
	unsigned int i, j, ii;
	unsigned int capabilities1, capabilities2, capabilities3, capabilities4;
	unsigned int license1;

	if (wal_init_worker(worker1)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		return -1;
	}

	if (wal_init_worker(worker2)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		return -1;
	}

	if (wal_init_worker(worker3)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		return -1;
	}

	if (wal_init_worker(worker4)) {
		xil_printf("\r\n ERROR: Init wrk failed.\r\n");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		return -1;
	}

	///////////////////////////////////////////////
	// Upgrade firmware
	for (i = 0; i < 3; i++){
		fill_FP1124P0_program_store[i] = fill_FA1124P0_program_store[i];
		fill_FP1124P1_program_store[i] = fill_FA1124P1_program_store[i];
	}
	for (i = EDKDSP_BOOT; i < WAL_FW_FULL_SIZE; i++){
		fill_FP1124P0_program_store[i] = fill_FA1124P0_program_store[i];
		fill_FP1124P1_program_store[i] = fill_FA1124P1_program_store[i];
	}

	///////////////////////////////////////////////
	//Program and initalize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker1, WAL_PBID_P0, fill_FP1124P0_program_store, -1);
	wal_set_firmware(worker1, WAL_PBID_P1, fill_FP1124P1_program_store, -1);

	wal_get_capabilities(worker1, WAL_PBID_P0, &capabilities1);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities1 = %x", capabilities1);
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;

	///////////////////////////////////////////////
	//Program and initalize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker2, WAL_PBID_P0, fill_FP1124P0_program_store, -1);
	wal_set_firmware(worker2, WAL_PBID_P1, fill_FP1124P1_program_store, -1);

	wal_get_capabilities(worker2, WAL_PBID_P0, &capabilities2);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities2 = %x", capabilities2);
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;

	///////////////////////////////////////////////
	//Program and initalize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker3, WAL_PBID_P0, fill_FP1124P0_program_store, -1);
	wal_set_firmware(worker3, WAL_PBID_P1, fill_FP1124P1_program_store, -1);

	wal_get_capabilities(worker3, WAL_PBID_P0, &capabilities3);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities3 = %x", capabilities3);
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;

	///////////////////////////////////////////////
	//Program and initalize the accelerator HW. Get HW Operations Code.
	wal_set_firmware(worker4, WAL_PBID_P0, fill_FP1124P0_program_store, -1);
	wal_set_firmware(worker4, WAL_PBID_P1, fill_FP1124P1_program_store, -1);

	wal_get_capabilities(worker4, WAL_PBID_P0, &capabilities4);
	xil_printf("\r\n MB0 : (EdkDSP 8xSIMD) Capabilities4 = %x", capabilities4);
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;

	mu[0] = (float) 0.01f; // LMS adaptation weight

	draw_line1(); // MB with FP  : Generating far-end signal ...

	///////////////////////////////////////////////
	// create sine wave input with offset -- with SAMPLE_RATE points per cycle
	for (i = 0; i < (8 * MM * NN + 8 * NN); i++) {
		ufd[i] = get_carrier(i);
	}

	///////////////////////////////////////////////
	// modulate the signal with a sine wave
	for (i = 0; i < (8 * MM * NN + 8 * NN); i++) {
		ufd[i] = ufd[i] * get_modulation(i);
	}

	///////////////////////////////////////////////
	// add noise
	for (i = 0; i < (8 * MM * NN + 8 * NN); i++) {
		ufd[i] = (float) 0.1 * (float) ufd[i] + (float) get_noise(i);
	}

	///////////////////////////////////////////////
	// initial data
	for (i = 0; i < 8 * NN; i++) {
		afmb[i] = (float) 0.1f * get_modulation(i);
	}

	afmb[(NN >> 1)] = 0.8f;

	///////////////////////////////////////////////
	// initial data
	for (i = 0; i < 8 * NN; i++) {
		af_mu_mb[i] = (float) 0.0f;
	}

	///////////////////////////////////////////////
	// initial data
	for (i = 0; i < 8 * NN; i++) {
		af_mu_pb[i] = (float) 0.0f;
	}

	///////////////////////////////////////////////
	// initial data
	for (i = 0; i < NN; i++) {
		j = 8 * NN - i - 1;
		bfmb[i] = ufd[j];
		bf_mu_mb[i] = ufd[j];
		bf_mu_pb[i] = ufd[j];
	}

	wal_get_license(worker1, WAL_PBID_P0, &license1);

	if (license1) {
		draw_line2(); // MB with BCE : Computing room response ...

		for (ii = 0; ii < (2 * MM); ii = ii + 2) {
			if (ii == MM) {
				for (i = 0; i < 8 * NN; i++) {
					afmb[i] = -0.1 * get_modulation(i);
				}
				afmb[2 * NN - (NN >> 1)] = (float) 0.8;

			}

			///////////////////////////////////////////////
			//pass control to CPU0 restart timer
			//pass control to CPU0
			if (ii == 2) {
				while (COMM_VAL == 0)
					;
				COMM_VAL = 0;
			}

			if (capabilities1 & WAL_BCE_JK_CAP_VPROD_S8) {
				///////////////////////////////////////////////
				// FIR by EdkDSP 8xSIMD
				bce_fp12_1x8_0_fir_pb_1x8_batch8(fw, WAL_PBID_P1, NN, ii, afmb,
						bfmb, ufd, &zfd[ii * 4 * NN]);
			} else {
				///////////////////////////////////////////////
				// FIR by MB with HW FP unit
				bce_fp12_1x8_0_fir_mb_1x8_batch8(NN, ii, afmb, bfmb, ufd,
						&zfd[ii * 4 * NN]);
			}

			///////////////////////////////////////////////
			//pass control to CPU0 measure and print mflops
			if (ii == 2) {
				while (COMM_VAL == 0)
					;
				COMM_VAL = 0;
			}
		}

		draw_line3(); // MB with FP  : Adding near-end signal ...

		for (i = 0; i < (8 * MM * NN + 8 * NN); i++) {
			zfd[i] = zfd[i] + 0.5f * get_noise2(i);
		}

		draw_line4(); // MB with BCE : LMS FIR filter identification ...
	}

	// Test of the HW evaluation license
	wal_get_license(worker1, WAL_PBID_P0, &license1);
	if (license1) {
		for (ii = 0; ii < (2 * MM); ii = ii + 2) {

			///////////////////////////////////////////////
			//pass control to CPU0 restart timer
			if (ii == 2) {
				while (COMM_VAL == 0)
					;
				COMM_VAL = 0;
			}

			if (capabilities1 & WAL_BCE_JK_CAP_VPROD_S8) {
				///////////////////////////////////////////////
				// LMS by EdkDSP 8xSIMD
				bce_fp12_1x8_0_lms_pb_1x8_batch8(fw, WAL_PBID_P0, NN, ii, ufd,
						&zfd[ii * 4 * NN], &mu[0], &af_mu_pb[0], &bf_mu_pb[0],
						&zf_mu_pb2[0], &zf_mu_pb3[0]);
			} else {
				///////////////////////////////////////////////
				// LMS by MB with HW FP unit
				bce_fp12_1x8_0_lms_mb_1x8_batch8(NN, ii, ufd, &zfd[ii * 4 * NN],
						&mu[0], &af_mu_pb[0], &bf_mu_pb[0], &zf_mu_pb2[0],
						&zf_mu_pb3[0]);
			}
			///////////////////////////////////////////////
			//pass control to CPU0 measure and print mflops
			if (ii == 2) {
				while (COMM_VAL == 0)
					;
				COMM_VAL = 0;
			}
		}

	} else {
		xil_printf(
				"\r\n\r\n Evaluation license will expire soon. Please contact:");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		xil_printf("\r\n UTIA AV CR, v.v.i.; Department of Signal Processing");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		xil_printf("\r\n Pod vodarenskou vezi 4");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		xil_printf("\r\n 18208, Prague 8, Czech republic");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
		xil_printf("\r\n http://zs.utia.cas.cz \r\n");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;
	}
	draw_line5(); // MB with FP  : LMS FIR filter identification ...

	for (ii = 0; ii < (2 * MM); ii = ii + 2) {

		///////////////////////////////////////////////
		//pass control to CPU0 restart timer
		if (ii == 2) {
			while (COMM_VAL == 0)
				;
			COMM_VAL = 0;
		}
		///////////////////////////////////////////////
		// Adaptive LMS filter on MicroBlaze
		bce_fp12_1x8_0_lms_mb_1x8_batch8(NN, ii, ufd, &zfd[ii * 4 * NN], &mu[0],
				&af_mu_mb[0], &bf_mu_mb[0], &zf_mu_mb2[0], &zf_mu_mb3[0]);

		///////////////////////////////////////////////
		//pass control to CPU0 measure and print mflops
		if (ii == 2) {
			while (COMM_VAL == 0)
				;
			COMM_VAL = 0;
		}
	}
	draw_line6(&zf_mu_pb2[0], &zf_mu_mb2[0], &zf_mu_pb3[0], &zf_mu_mb3[0]);

	xil_printf("\r\n");
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;
	return (0);
}

int main(void) {

	int Status;
	int worker;

	init_platform();

	irq_count = 0;

	///////////////////////////////////////////////
	// Initialize driver instance for IrqGen IRQ
	IrqGenInstancePtr.Config.DeviceId = IRQ_GEN_ID;
	IrqGenInstancePtr.Config.BaseAddress = IRQ_PCORE_GEN_BASE;
	IrqGenInstancePtr.IsReady = XIL_COMPONENT_IS_READY;
	IrqGenInstancePtr.IsStarted = 0;

	///////////////////////////////////////////////
	// Setup the interrupts such that interrupt processing can occur.
	Status = SetupInterruptSystem();
	if (Status != XST_SUCCESS) {
		return XST_FAILURE;
	}

	///////////////////////////////////////////////
	// wait until CPU0 hands off flag
	// return control to CPU0
	while (COMM_VAL == 0)
		;
	COMM_VAL = 0;

	worker = 1;
	///////////////////////////////////////////////
	// Loop forever
	while (1) {
		///////////////////////////////////////////////
		// wait until CPU0 hands off flag
		// compute with the possibility if the xil_printf from MB0 :
		while (COMM_VAL == 0)
			;
		xil_printf("\r\n\r\n MB0 : (EdkDSP 8xSIMD) Write firmware ... ");
		while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
			;

		///////////////////////////////////////////////
		// compute edkdsp_fir_lms on MB0 :
		Status = edkdsp_fir_lms();
		if (Status != XST_SUCCESS) {
			return XST_FAILURE;
		}

		///////////////////////////////////////////////
		// compute edkdsp_eval_op on MB0 :
		Status = edkdsp_eval_op(worker);
		if (Status != XST_SUCCESS) {
			return XST_FAILURE;
		}

		///////////////////////////////////////////////
		// rotation of tested workers on MB0 :
		if (worker < 4){
			worker = worker + 1;
		} else {
			worker = 1;
		}

		///////////////////////////////////////////////
		// pass control to CPU0
		COMM_VAL = 0;
	}

	///////////////////////////////////////////////
	// wait for an HW interrupt, not used in this demo
	while (irq_count == 0)
		;

	xil_printf("MB0 : IRQ Acknowledged\n\r");
	irq_count = 0;

	///////////////////////////////////////////////
	// Wait until UART TX FIFO is empty
	while ((Xil_In32(UART_BASE + 0x2C) & 0x08) != 0x08)
		;

	///////////////////////////////////////////////
	//pass control to CPU0
	COMM_VAL = 0;

	cleanup_platform();

	return XST_SUCCESS;
}

/****************************************************************************/
/**
 * This function contains the low level code to write a character to the PS
 * UART
 *
 * @param	BaseAddress contains the base address of the PS UART
 * @param	Data contains the value to write to the UART tx fifo
 *
 * @return	None.
 *
 * @note		None.
 *
 *****************************************************************************/
void XUartChanged_SendByte(u32 BaseAddress, u8 Data) {
	// Wait until there is space in TX FIFO
	while ((Xil_In32((BaseAddress + 0x2C) & 0x10) ) == 0x10)
		;

	// Write the byte into the TX FIFO
	Xil_Out32((BaseAddress + 0x30), Data);
}

/****************************************************************************/
/**
 * This function is used by xil_printf() to print a character to stdout;
 * The BSP was not configured for stdout since the PS UART is not visible
 * by the BSP.
 *
 * @param	c contains the value to print
 *
 * @return	None.
 *
 * @note		None.
 *
 *****************************************************************************/
void outbyte(char c) {
	XUartChanged_SendByte(UART_BASE, c);
}

/****************************************************************************/
/**
 * This function sets up the interrupt system for the example.  The processing
 * contained in this funtion assumes the hardware system was built with
 * and interrupt controller.
 *
 * @param	None.
 *
 * @return	A status indicating XST_SUCCESS or a value that is contained in
 *		xstatus.h.
 *
 * @note		None.
 *
 *****************************************************************************/
int SetupInterruptSystem() {
	int Result;
	INTC *IntcInstancePtr = &Intc;

	/*
	 * Initialize the interrupt controller driver so that it's ready to use.
	 * specify the device ID that was generated in xparameters.h
	 */
	Result = XIntc_Initialize(IntcInstancePtr, INTC_DEVICE_ID);
	if (Result != XST_SUCCESS) {
		return Result;
	}

	/* Hook up interrupt service routine */
#ifdef FAST_IRQ_EN
	XIntc_ConnectFastHandler(IntcInstancePtr, IRQ_GEN_INTERRUPT_ID,
			(XFastInterruptHandler) IrqGenIntrHandler);
#else
	XIntc_Connect(IntcInstancePtr, IRQ_GEN_INTERRUPT_ID,
			(Xil_ExceptionHandler)IrqGenIntrHandler, &IrqGenInstancePtr);
#endif

	/* Enable the interrupt vector at the interrupt controller */

	XIntc_Enable(IntcInstancePtr, IRQ_GEN_INTERRUPT_ID);

	/*
	 * Start the interrupt controller such that interrupts are recognized
	 * and handled by the processor
	 */
	Result = XIntc_Start(IntcInstancePtr, XIN_REAL_MODE);
	if (Result != XST_SUCCESS) {
		return Result;
	}

	/*
	 * Initialize the exception table and register the interrupt
	 * controller handler with the exception table
	 */
	Xil_ExceptionInit();

	Xil_ExceptionRegisterHandler(XIL_EXCEPTION_ID_INT,
			(Xil_ExceptionHandler) INTC_HANDLER, IntcInstancePtr);

	/* Enable non-critical exceptions */
	Xil_ExceptionEnable();

	return XST_SUCCESS;
}

/*****************************************************************************/
/**
 *
 * This function is the Interrupt handler for the PL Interrupt.
 * It is called when the PL creates an interrupt and the interrupt gets serviced.
 *
 * This function sets the global varialbe irq_count=1 and clears the interrupt source.
 *
 * @param	CallBackRef is a pointer to the callback function.
 *
 * @return	None.
 *
 * @note		None.
 *
 ******************************************************************************/
static void IrqGenIntrHandler(void *CallBackRef) {

//	XIrqGenIrq *InstancePtr = (XIrqGenIrq *)CallBackRef;

	/*
	 * Clear the interrupt source
	 */
	Xil_Out32(IrqGenInstancePtr.Config.BaseAddress, 0);
//	Xil_Out32(InstancePtr->Config.BaseAddress, 0);

	irq_count = 1;

}

